@@ -12,6 +12,8 @@
 #include <Wt/WJavaScript>
 #include <Wt/WSound>
 
+#include "SimpleChatServer.h"
+
 namespace Wt {
   class WApplication;
   class WPushButton;
@@ -20,7 +22,6 @@ namespace Wt {
   class WTextArea;
 }
 
-class SimpleChatServer;
 class ChatEvent;
 
 /**
@@ -30,7 +31,8 @@ class ChatEvent;
 
 /*! \brief A self-contained chat widget.
  */
-class SimpleChatWidget : public Wt::WContainerWidget
+class SimpleChatWidget : public Wt::WContainerWidget,
+			 public SimpleChatServer::Client
 {
 public:
   /*! \brief Create a chat widget that will connect to the given server.
@@ -41,6 +43,10 @@ public:
    */
   ~SimpleChatWidget();
 
+  void connect();
+  void disconnect();
+
+
   /*! \brief Show a simple login screen.
    */
   void letLogin();
@@ -57,20 +63,27 @@ public:
 
   int userCount() { return users_.size(); }
 
+  const Wt::WString& userName() const { return user_; }
+
 protected:
   virtual void createLayout(Wt::WWidget *messages, Wt::WWidget *userList,
 			    Wt::WWidget *messageEdit,
 			    Wt::WWidget *sendButton, Wt::WWidget *logoutButton);
 
   virtual void updateUsers();
+  virtual void newMessage();
 
   virtual void render(Wt::WFlags<Wt::RenderFlag> flags);
 
+protected:
+  bool loggedIn() const;
+
 private:
   typedef std::map<Wt::WString, bool> UserMap;
   UserMap users_;
 
   SimpleChatServer&     server_;
+  bool                  loggedIn_;
 
   Wt::JSlot             clearInput_;
 
@@ -90,7 +103,6 @@ private:
   void login();
   void send();
   void updateUser();
-  bool loggedIn() const;
 
   /* called from another session */
   void processChatEvent(const ChatEvent& event);
