@@ -20,7 +20,7 @@ public:
 
 protected:
   virtual WWidget *examples();
-  virtual WWidget *download();
+  virtual WWidget *createQuoteForm();
   virtual WWidget *sourceViewer(const std::string& deployPath);
   virtual std::string filePrefix() const { return "jwt-"; }
 
@@ -33,6 +33,9 @@ private:
   WWidget *chartExample();
   WWidget *composerExample();
   WWidget *treeviewExample();
+  WWidget *chatExample();
+  WWidget *figtreeExample();
+  WWidget *widgetGalleryExample();
 
   std::string jwtExamplePath_;
 };
