@@ -108,6 +108,18 @@ void InitSchema::actPtr(const PtrRef<C>& field)
   }
 }
 
+template<class C>
+void InitSchema::actWeakPtr(const WeakPtrRef<C>& field)
+{
+  const char *joinTableName = session_.tableName<C>();
+  std::string joinName = field.joinName();
+  if (joinName.empty())
+    joinName = mapping_.tableName;
+
+  mapping_.sets.push_back
+    (Session::SetInfo(joinTableName, ManyToOne, joinName, std::string(), 0));
+}
+
 template<class C>
 void InitSchema::actCollection(const CollectionRef<C>& field)
 {
@@ -150,6 +162,19 @@ template<class C>
 void DropSchema::actPtr(const PtrRef<C>& field)
 { }
 
+template<class C>
+void DropSchema::actWeakPtr(const WeakPtrRef<C>& field)
+{ 
+  const char *tableName = session_.tableName<C>();
+  if (tablesDropped_.count(tableName) == 0) {
+    DropSchema action(session_, 
+		      *session_.getMapping(tableName), 
+		      tablesDropped_);
+    C dummy;
+    action.visit(dummy);
+  }
+}
+
 template<class C>
 void DropSchema::actCollection(const CollectionRef<C>& field)
 {
@@ -172,6 +197,24 @@ void DropSchema::actCollection(const CollectionRef<C>& field)
      * DboAction
      */
 
+template<class C>
+void DboAction::actWeakPtr(const WeakPtrRef<C>& field)
+{
+  Session::SetInfo *setInfo = &mapping_->sets[setIdx_++];
+
+  if (dbo_->session()) {
+    int statementIdx = Session::FirstSqlSelectSet + setStatementIdx_;
+
+    const std::string& sql
+      = dbo_->session()->getStatementSql(mapping_->tableName, statementIdx);
+
+    field.value().setRelationData(dbo_, &sql, setInfo);
+  } else
+    field.value().setRelationData(dbo_, 0, setInfo);
+
+  setStatementIdx_ += 1;
+}
+
 template<class C>
 void DboAction::actCollection(const CollectionRef<C>& field)
 {
@@ -293,6 +336,23 @@ void SaveBaseAction::actPtr(const PtrRef<C>& field)
   }
 }
 
+template<class C>
+void SaveBaseAction::actWeakPtr(const WeakPtrRef<C>& field)
+{
+  switch (pass_) {
+  case Dependencies:
+    break;
+
+  case Self:
+    if (isInsert_)
+      needSetsPass_ = true;
+
+    break;
+  case Sets:
+    DboAction::actWeakPtr(field);
+  }
+}
+
 template<class C>
 void SaveBaseAction::actCollection(const CollectionRef<C>& field)
 {
@@ -500,6 +560,13 @@ template<class C>
 void TransactionDoneAction::actPtr(const PtrRef<C>& field)
 { }
 
+template<class C>
+void TransactionDoneAction::actWeakPtr(const WeakPtrRef<C>& field)
+{
+  if (!success_)
+    DboAction::actWeakPtr(field);
+}
+
 template<class C>
 void TransactionDoneAction::actCollection(const CollectionRef<C>& field)
 {
@@ -562,6 +629,12 @@ void SessionAddAction::actCollection(const CollectionRef<C>& field)
   // FIXME: cascade add ?
 }
 
+template<class C>
+void SessionAddAction::actWeakPtr(const WeakPtrRef<C>& field)
+{
+  DboAction::actWeakPtr(field);
+}
+
     /*
      * SetReciproceAction
      */
@@ -596,6 +669,11 @@ void SetReciproceAction::actPtr(const PtrRef<C>& field)
     field.value().reset(value_);
 }
 
+template<class C>
+void SetReciproceAction::actWeakPtr(const WeakPtrRef<C>& field)
+{
+}
+
 template<class C>
 void SetReciproceAction::actCollection(const CollectionRef<C>& field)
 {
@@ -676,10 +754,13 @@ void ToAnysAction::actPtr(const PtrRef<C>& field)
   field.visit(*this, 0);
 }
 
+template<class C>
+void ToAnysAction::actWeakPtr(const WeakPtrRef<C>& field)
+{ }
+
 template<class C>
 void ToAnysAction::actCollection(const CollectionRef<C>& field)
-{
-}
+{ }
 
     /*
      * FromAnyAction
@@ -690,7 +771,8 @@ void FromAnyAction::visit(const ptr<C>& obj)
 {
   if (dbo_traits<C>::surrogateIdField()) {
     if (index_ == 0)
-      throw Exception("dbo_result_traits::setValues(): cannot set surrogate id.");
+      throw Exception("dbo_result_traits::setValues(): cannot set surrogate "
+		      "id.");
     --index_;
   }
 
@@ -753,10 +835,13 @@ void FromAnyAction::actPtr(const PtrRef<C>& field)
   field.visit(*this, 0);
 }
 
+template<class C>
+void FromAnyAction::actWeakPtr(const WeakPtrRef<C>& field)
+{ }
+
 template<class C>
 void FromAnyAction::actCollection(const CollectionRef<C>& field)
-{
-}
+{ }
 
   }
 }
