@@ -182,6 +182,7 @@ void QueryModel<Result>::invalidateData()
   cachedRowCount_ = cacheStart_ = currentRow_ = -1;
   cache_.clear();
   rowValues_.clear();
+  stableIds_.clear();
 }
 
 template <class Result>
@@ -207,6 +208,17 @@ void QueryModel<Result>::sort(int column, SortOrder order)
   dataReloaded();
 }
 
+template <class Result>
+Result QueryModel<Result>::stableResultRow(int row) const
+{
+  StableResultIdMap::const_iterator i = stableIds_.find(row);
+
+  if (i != stableIds_.end())
+    return resultById(i->second);
+  else
+    return resultRow(row);
+}
+
 template <class Result>
 Result& QueryModel<Result>::resultRow(int row)
 {
@@ -230,6 +242,12 @@ Result& QueryModel<Result>::resultRow(int row)
     cache_.clear();
     cache_.insert(cache_.end(), results.begin(), results.end());   
 
+    for (unsigned i = 0; i < cache_.size(); ++i) {
+      long long id = resultId(cache_[i]);
+      if (id != -1)
+	stableIds_[cacheStart_ + i] = id;
+    }
+
     if (row >= cacheStart_ + static_cast<int>(cache_.size()))
       throw Exception("QueryModel: geometry inconsistent with database");
 
@@ -409,6 +427,20 @@ boost::any QueryModel<Result>::headerData(int section, Orientation orientation,
     return WAbstractTableModel::headerData(section, orientation, role);
 }
 
+template <class Result>
+long long QueryModel<Result>::resultId(const Result& result) const
+{
+  return query_result_traits<Result>::id(result);
+}
+
+template <class Result>
+Result QueryModel<Result>::resultById(long long id) const
+{
+  Transaction transaction(query_.session());
+
+  return query_result_traits<Result>::findById(query_.session(), id);
+}
+
   }
 }
 
