@@ -7,12 +7,12 @@
 #ifndef WT_DBO_QUERY_IMPL_H_
 #define WT_DBO_QUERY_IMPL_H_
 
-#include <stdexcept>
 #include <boost/tuple/tuple.hpp>
 
 #include <Wt/Dbo/Exception>
 #include <Wt/Dbo/Field>
 #include <Wt/Dbo/SqlStatement>
+#include <Wt/Dbo/DbAction>
 
 #include <Wt/Dbo/Field_impl.h>
 
@@ -28,7 +28,8 @@ completeQuerySelectSql(const std::string& sql,
 		       const std::string& groupBy,
 		       const std::string& orderBy,
 		       int limit, int offset,
-		       const std::vector<FieldInfo>& fields);
+		       const std::vector<FieldInfo>& fields,
+		       bool useRowsFromTo);
 
 extern std::string WTDBO_API
 createQuerySelectSql(const std::string& from,
@@ -36,7 +37,8 @@ createQuerySelectSql(const std::string& from,
 		     const std::string& groupBy,
 		     const std::string& orderBy,
 		     int limit, int offset,
-		     const std::vector<FieldInfo>& fields);
+		     const std::vector<FieldInfo>& fields,
+		     bool useRowsFromTo);
 
 extern std::string WTDBO_API
 createWrappedQueryCountSql(const std::string& query);
@@ -47,13 +49,14 @@ createQueryCountSql(const std::string& query,
 		    const std::string& where,
 		    const std::string& groupBy,
 		    const std::string& orderBy,
-		    int limit, int offset);
+		    int limit, int offset,
+		    bool useRowsFromTo);
 
 extern void WTDBO_API
 substituteFields(const SelectFieldList& list,
-		      const std::vector<FieldInfo>& fs,
-		      std::string& sql,
-		      int offset);
+		 const std::vector<FieldInfo>& fs,
+		 std::string& sql,
+		 int offset);
 
 extern void WTDBO_API 
 parseSql(const std::string& sql, SelectFieldLists& fieldLists,
@@ -77,7 +80,8 @@ QueryBase<Result>::QueryBase(Session& session, const std::string& table,
 			     const std::string& where)
   : session_(&session)
 {
-  sql_ = "from \"" + table + "\" " + where;
+  sql_ = "from " + table + ' ' + where;
+
   simpleCount_ = true;
 }
 
@@ -127,12 +131,14 @@ QueryBase<Result>::statements(const std::string& where,
 
     std::vector<FieldInfo> fs = this->fields();
     sql = Impl::createQuerySelectSql(sql_, where, groupBy, orderBy,
-				     limit, offset, fs);
+				     limit, offset, fs,
+				     this->session_->useRowsFromTo_);
     statement = this->session_->getOrPrepareStatement(sql);
 
     if (simpleCount_)
       sql = Impl::createQueryCountSql(sql, sql_, where, groupBy, orderBy,
-				      limit, offset);
+				      limit, offset,
+				      this->session_->useRowsFromTo_);
     else
       sql = Impl::createWrappedQueryCountSql(sql);
 
@@ -155,14 +161,16 @@ QueryBase<Result>::statements(const std::string& where,
     }
 
     sql = Impl::completeQuerySelectSql(sql, where, groupBy, orderBy,
-				       limit, offset, fs);
+				       limit, offset, fs,
+				       this->session_->useRowsFromTo_);
 
     statement = this->session_->getOrPrepareStatement(sql);
 
     if (simpleCount_) {
       std::string from = sql_.substr(selectFieldLists_.front().back().end);
       sql = Impl::createQueryCountSql(sql, from, where, groupBy, orderBy,
-				      limit, offset);
+				      limit, offset,
+				      this->session_->useRowsFromTo_);
     } else
       sql = Impl::createWrappedQueryCountSql(sql);
 
@@ -184,7 +192,7 @@ void QueryBase<Result>::fieldsForSelect(const SelectFieldList& list,
 
   query_result_traits<Result>::getFields(*session_, &aliases, result);
   if (!aliases.empty())
-    throw std::logic_error("Session::query(): too many aliases for result");
+    throw Exception("Session::query(): too many aliases for result");
 }
 
 template <class Result>
@@ -394,10 +402,12 @@ template <class Result>
 Query<Result, DynamicBinding>&
 Query<Result, DynamicBinding>::where(const std::string& where)
 {
-  if (!where_.empty())
-    where_ += " and ";
+  if (!where.empty()) {
+    if (!where_.empty())
+      where_ += " and ";
 
-  where_ += "(" + where + ")";
+    where_ += "(" + where + ")";
+  }
 
   return *this;
 }
@@ -491,19 +501,29 @@ template <class Result>
 void Query<Result, DynamicBinding>::bindParameters(SqlStatement *statement)
   const
 {
-  SaveBaseAction binder(statement, 0);
+  SaveBaseAction binder(this->session_, statement, 0);
 
   for (unsigned i = 0; i < parameters_.size(); ++i)
     parameters_[i]->bind(binder);
 
-  if (limit_ != -1) {
-    int v = limit_;
-    field(binder, v, "limit");
-  }
+  if (!this->session_->useRowsFromTo_) {
+    if (limit_ != -1) {
+      int v = limit_;
+      field(binder, v, "limit");
+    }
 
-  if (offset_ != -1) {
-    int v = offset_;
-    field(binder, v, "offset");
+    if (offset_ != -1) {
+      int v = offset_;
+      field(binder, v, "offset");
+    }
+  } else {
+    if (limit_ != -1 || offset_ != -1) {
+      int from = offset_ == -1 ? 1 : offset_ + 1;
+      field(binder, from, "from");
+
+      int to = (limit_ == -1) ? (1 << 30) : (from + limit_ - 1);
+      field(binder, to, "to");
+    }
   }
 }
 
