@@ -8,6 +8,7 @@
 #define FONT_SUPPORT_H_
 
 #ifndef HAVE_PANGO
+#include <list>
 #include <string>
 #else
 #include <pango/pango.h>
@@ -27,6 +28,7 @@ namespace Wt {
 class FontSupport
 {
 public:
+#ifndef WT_TARGET_JAVA
   class Bitmap {
   public:
     Bitmap(int width, int height);
@@ -45,6 +47,7 @@ public:
 
     unsigned char *buffer_;
   };
+#endif
 
   class FontMatch {
   public:
@@ -64,7 +67,9 @@ public:
     bool matched() const { return quality_ > 0; }
 
     std::string fileName() const { return file_; }
+    void setFileName(const std::string &file) { file_ = file; }
     double quality() const { return quality_; }
+    void setQuality(double quality) { quality_ = quality; }
 
 #endif
 
@@ -128,6 +133,7 @@ public:
    */
   bool canRender() const;
 
+#ifndef WT_TARGET_JAVA
   /*
    * Draws the text, using pango and libfreetype to do the actual rendering
    *
@@ -138,6 +144,7 @@ public:
 		const WTransform& transform,
 		Bitmap& bitmap,
 		WFlags<AlignmentFlag> alignmentFlags, const WString& text);
+#endif
 
   /*
    * If libpango support is available, this is a no-op.
@@ -171,6 +178,16 @@ private:
 
   std::vector<FontCollection> fontCollections_;
 
+  struct Matched {
+    WFont font;
+    FontMatch match;
+
+    Matched() : font(), match() { }
+  };
+
+  typedef std::list<Matched> MatchCache;
+  mutable MatchCache cache_;
+
   const WFont *font_;
 
   FontMatch matchFont(const WFont& font, const std::string& directory,
