@@ -14,6 +14,7 @@
 
 #include "Reply.h"
 #include "../web/Configuration.h"
+#include "../web/WebRequest.h"
 
 namespace http {
 namespace server {
@@ -29,8 +30,6 @@ typedef boost::shared_ptr<WtReply> WtReplyPtr;
 class WtReply : public Reply
 {
 public:
-  typedef boost::function<void(void)> CallbackFunction;
-
   WtReply(const Request& request, const Wt::EntryPoint& ep,
 	  const Configuration &config);
   ~WtReply();
@@ -47,38 +46,42 @@ public:
   void setContentLength(::int64_t length);
   void setContentType(const std::string& type);
   void setLocation(const std::string& location);
-  void send(const std::string& text, CallbackFunction callBack,
+  void send(const Wt::WebRequest::WriteCallback& callBack,
 	    bool responseComplete);
-  void readWebSocketMessage(CallbackFunction callBack);
+  void readWebSocketMessage(const Wt::WebRequest::ReadCallback& callBack);
   bool readAvailable();
 
   virtual bool waitMoreData() const;
 
-  std::istream& cin() { return *cin_; }
+  std::istream& in() { return *in_; }
+  std::ostream& out() { return out_; }
   const Request& request() const { return request_; }
   std::string urlScheme() const { return urlScheme_; }
 
 protected:
   const Wt::EntryPoint& entryPoint_;
-  std::iostream    *cin_;
-  std::stringstream cin_mem_;
-  std::string       requestFileName_;
-  std::string       cout_, nextCout_;
-  std::string       contentType_;
-  std::string       location_;
-  std::string       urlScheme_;
-  bool              responseSent_;
-  ::int64_t         contentLength_, bodyReceived_;
-  bool              sendingMessages_;
-  CallbackFunction  fetchMoreDataCallback_, readMessageCallback_;
-  HTTPRequest      *httpRequest_;
-  bool              sending_;
+  std::iostream *in_;
+  std::stringstream in_mem_;
+  std::string requestFileName_;
+  boost::asio::streambuf out_buf_;
+  std::ostream out_;
+  std::string contentType_;
+  std::string location_;
+  std::string urlScheme_;
+  std::size_t sending_;
+  ::int64_t contentLength_, bodyReceived_;
+  bool sendingMessages_;
+  Wt::WebRequest::WriteCallback fetchMoreDataCallback_;
+  Wt::WebRequest::ReadCallback readMessageCallback_;
+  HTTPRequest *httpRequest_;
 
-  virtual std::string     contentType();
-  virtual std::string     location();
-  virtual ::int64_t       contentLength();
+  char gatherBuf_[16];
 
-  virtual asio::const_buffer nextContentBuffer();  
+  virtual std::string contentType();
+  virtual std::string location();
+  virtual ::int64_t contentLength();
+
+  virtual void nextContentBuffers(std::vector<asio::const_buffer>& result);
 
 private:
   void readRestWebSocketHandshake();
