@@ -50,7 +50,7 @@ WT_DECLARE_WT_MEMBER
 			      be remeasured */
    var layoutDirty = true; /* all items dirty need to be remeasured */
    var topLevel = false, parent = null, parentItemWidget = null,
-     parentInitialized = false, parentMargin = [], parentMinPW = 0;
+     parentInitialized = false, parentMargin = [], parentWithWtPS = false;
 
    var rtl = $(document.body).hasClass('Wt-rtl');
 
@@ -62,6 +62,7 @@ WT_DECLARE_WT_MEMBER
 	 maxSize: maxWidth,
 	 measures: [],
 	 sizes: [],
+	 stretched: [],
 	 fixedSize: [],
 	 Left: (rtl ? 'Right':'Left'),
 	 left: (rtl ? 'right':'left'),
@@ -83,6 +84,7 @@ WT_DECLARE_WT_MEMBER
 	 maxSize: maxHeight,
 	 measures: [],
 	 sizes: [],
+	 stretched: [],
 	 fixedSize: [],
 	 Left: 'Top',
 	 left: 'top',
@@ -101,23 +103,35 @@ WT_DECLARE_WT_MEMBER
 
    jQuery.data(document.getElementById(id), 'layout', this);
 
-   function calcPreferredSize(element, dir) {
+   function calcPreferredSize(element, dir, asSet) {
      var DC = DirConfig[dir];
+
+     /* Allow accurate measurement for widgets with offsets */
+     var l = element.style[DC.left];
+     setCss(element, DC.left, NA_px);
      var scrollWidth = dir ? element.scrollHeight : element.scrollWidth;
      var clientWidth = dir ? element.clientHeight : element.clientWidth;
      var offsetWidth = dir ? element.offsetHeight : element.offsetWidth;
+     setCss(element, DC.left, l);
+
 
      /*
       * Firefox adds the -NA offset to the reported width ??
       */
-     if (clientWidth >= NA) {
+     if (clientWidth >= NA)
        clientWidth -= NA;
-       if (scrollWidth >= NA)
-	 scrollWidth -= NA;
-     }
+     if (scrollWidth >= NA)
+       scrollWidth -= NA;
+     if (offsetWidth >= NA)
+       offsetWidth -= NA;
 
      if (scrollWidth === 0) {
        scrollWidth = WT.pxself(element, DC.size);
+
+       if (scrollWidth !== 0 && !WT.isOpera && !WT.isGecko)
+	 scrollWidth -=
+	   WT.px(element, 'border' + DC.Left + 'Width') +
+	   WT.px(element, 'border' + DC.Right + 'Width');
      }
 
      if (scrollWidth === clientWidth) {
@@ -131,7 +145,33 @@ WT_DECLARE_WT_MEMBER
        scrollWidth = clientWidth;
      }
 
-     if (!WT.isOpera && !WT.isGecko)
+     if (scrollWidth > offsetWidth) {
+       /*
+	* could be because of a nested absolutely positioned element
+	* with z-index > 0 ?, or could be because of genuine contents that is
+	* not fitting the parent.
+	*
+	* the latter case is only possible if a size is currently set on the
+	* element
+	*/
+       if (WT.pxself(element, DC.size) == 0)
+	 scrollWidth = 0;
+       else {
+	 var visiblePopup = false;
+	 $(element).find(".Wt-popup").each
+	   (function(index) {
+	     if (this.style.display != 'none')
+	       visiblePopup = true;
+	   });
+	 if (visiblePopup)
+	   scrollWidth = 0;
+       }
+     }
+
+     if (asSet)
+       return scrollWidth;
+
+     if (!WT.isOpera)
        scrollWidth +=
 	 WT.px(element, 'border' + DC.Left + 'Width') +
 	 WT.px(element, 'border' + DC.Right + 'Width');
@@ -149,7 +189,11 @@ WT_DECLARE_WT_MEMBER
      if (scrollWidth < offsetWidth)
        scrollWidth = offsetWidth;
 
-     return scrollWidth;
+     var maxSize = WT.px(element, 'max' + DC.Size);
+     if (maxSize > 0)
+       scrollWidth = Math.min(maxSize, scrollWidth);
+
+     return Math.round(scrollWidth);
    }
 
    function calcMinimumSize(element, dir) {
@@ -220,12 +264,12 @@ WT_DECLARE_WT_MEMBER
    function boxMargin(el, dir) {
      var DC = DirConfig[dir];
 
-     return WT.px(el, 'border' + DC.Left + 'Width') +
-       WT.px(el, 'border' + DC.Right + 'Width') +
-       WT.px(el, 'margin' + DC.Left) +
-       WT.px(el, 'margin' + DC.Right) +
-       WT.px(el, 'padding' + DC.Left) +
-       WT.px(el, 'padding' + DC.Right);
+     return Math.round(WT.px(el, 'border' + DC.Left + 'Width') +
+		       WT.px(el, 'border' + DC.Right + 'Width') +
+		       WT.px(el, 'margin' + DC.Left) +
+		       WT.px(el, 'margin' + DC.Right) +
+		       WT.px(el, 'padding' + DC.Left) +
+		       WT.px(el, 'padding' + DC.Right));
    }
 
    function setItemDirty(item, scheduleAdjust) {
@@ -270,7 +314,7 @@ WT_DECLARE_WT_MEMBER
      var preferredSize = [], minimumSize = [],
        totalPreferredSize = 0, totalMinSize = 0, di, oi;
 
-     var measurePreferredForStretching = true;
+     var measurePreferredForStretching = true, spanned = false;
 
      for (di = 0; di < dirCount; ++di) {
        var dPreferred = 0;
@@ -280,21 +324,27 @@ WT_DECLARE_WT_MEMBER
        for (oi = 0; oi < otherCount; ++oi) {
 	 var item = DC.getItem(di, oi);
 	 if (item) {
-	   if (!item.w) {
+	   if (!item.w || (dir == HORIZONTAL && item.dirty)) {
 	     var $w = $("#" + item.id);
-	     item.w = $w.get(0);
-	     (function() { var citem = item;
-	       $w.find("img").load(function() { setItemDirty(citem); });
-	     })();
+	     var w2 = $w.get(0);
+	     if (w2 != item.w) {
+	       item.w = w2;
+	       $w.find("img").add($w.filter("img"))
+		 .bind('load',
+		       { item: item},
+		       function(event) {
+			 setItemDirty(event.data.item, true);
+		       });
 
-	     item.w.style[DC.left] = item.w.style[OC.left] = NA_px;
+	       item.w.style[DC.left] = item.w.style[OC.left] = NA_px;
+	     }
 	   }
 
 	   if (!progressive && item.w.style.position != 'absolute') {
 	     item.w.style.position = 'absolute';
 	     item.w.style.visibility = 'hidden';
 
-	     if (!item.w.wtResize) {
+	     if (!item.w.wtResize && !WT.isIE) {
 	       item.w.style.boxSizing = 'border-box';
 	       var cssPrefix = WT.cssPrefix('BoxSizing');
 	       if (cssPrefix)
@@ -317,6 +367,13 @@ WT_DECLARE_WT_MEMBER
 	   if (!item.fs)
 	     item.fs = []; // fixed size (size defined by inline size or CSS)
 
+	   if ($(item.w).hasClass('Wt-hidden')) {
+	     item.ps[dir] = item.ms[dir] = 0;
+	     continue;
+	   }
+
+	   var first = !item.set;
+
 	   if (!item.set)
 	     item.set = [false, false];
 
@@ -339,16 +396,19 @@ WT_DECLARE_WT_MEMBER
 		* account the size set for a widget by CSS. But we can't really
 		* read this -- computedStyle for width or height measures
 		* instead of interpreting the stylesheet ... !
+		*
+		* we'll consider computedStyle only for vertical size, and only
+		* the first time
 		*/
                if (!item.set[dir]) {
-		 if (dir == HORIZONTAL) {
+		 if (dir == HORIZONTAL || !first) {
 		   var fw = WT.pxself(item.w, DC.size);
 		   if (fw)
 		     item.fs[dir] = fw + margin(item.w, dir);
 		   else
 		     item.fs[dir] = 0;
 		 } else {
-		   var fw = WT.px(item.w, DC.size);
+		   var fw = Math.round(WT.px(item.w, DC.size));
 		   if (fw > Math.max(sizePadding(item.w, dir), wMinimum))
 		     item.fs[dir] = fw + margin(item.w, dir);
 		   else
@@ -364,42 +424,79 @@ WT_DECLARE_WT_MEMBER
 		   || (DC.config[di][STRETCH] <= 0)) {
 
 		 if (!item.layout) {
-		   var calculated = calcPreferredSize(item.w, dir);
+		   if (item.wasLayout) {
+		     item.wasLayout = false;
+		     item.set = [false, false];
+		     item.ps -= 0.1; 
+		     if (item.w.wtResize)
+		       item.w.wtResize(item.w, -1, -1, true);
+		     setCss(item.w, DirConfig[1].size, '');
+		   }
+
+		   var calculated = calcPreferredSize(item.w, dir, false);
 
 		   /*
 		    * If we've set the size then we should not take the
 		    * set size as the preferred size, instead we revert
 		    * to a previous preferred size.
-		    *
-		    * If this is an item that is stretching, then we should
-		    * not remeasure the preferred size since it might confuse
-		    * the user with constant resizing
 		    */
-		   if ((typeof item.ps[dir] === 'undefined'
-			|| DC.config[di][STRETCH] <= 0)
-		     && (!item.set[dir] || (calculated != item.psize[dir])))
-		     wPreferred = Math.max(wPreferred, calculated);
-		   else
+		   var sizeSet = item.set[dir];
+		   if (sizeSet) {
+		     /*
+		      * Make an exception for very small sizes set ...
+		      * Typically to discount for margin/paddings on
+		      * form elements which are not properly measured
+		      * if the item is being reduced to very small
+		      * width/height
+		      */
+		     if (item.psize[dir] > 8)
+		       sizeSet = (calculated >= item.psize[dir] - 2) &&
+			 (calculated <= item.psize[dir] + 2);
+		   }
+
+		   /*
+		    * If this is an item that is stretching and has
+		    * been stretched (item.set[dir]), then we should
+		    * not remeasure the preferred size since it might
+		    * confuse the user with constant resizing.
+		    *
+		    * -- FIXME: what if sum stretch=0, we are actually
+		    * also stretching ?
+		    */
+		   var stretching = (typeof item.ps[dir] !== 'undefined')
+		     && (DC.config[di][STRETCH] > 0)
+		     && item.set[dir];
+
+		   if (sizeSet || stretching)
 		     wPreferred = Math.max(wPreferred, item.ps[dir]);
+		   else
+		     wPreferred = Math.max(wPreferred, calculated);
 
 		   item.ps[dir] = wPreferred;
 		 } else {
-		   wPreferred = Math.max(wPreferred, item.ps[dir]);
+		   if (wPreferred == 0)
+		     wPreferred = item.ps[dir];
 		   item.ps[dir] = wPreferred;
 		 }
-	       } else if (item.layout)
-		 wPreferred = Math.max(wPreferred, item.ps[dir]);
 
-	       if (!item.span || item.span[dir] == 1)
+	       } else if (item.layout) {
+		 if (wPreferred == 0)
+		   wPreferred = item.ps[dir];
+	       }
+
+	       if (!item.span || item.span[dir] == 1) {
 		 if (wPreferred > dPreferred)
 		   dPreferred = wPreferred;
+	       } else
+		 spanned = true;
 	     } else {
 	       if (!item.span || item.span[dir] == 1) {
 		 if (item.ps[dir] > dPreferred)
 		   dPreferred = item.ps[dir];
 		 if (item.ms[dir] > dMinimum)
 		   dMinimum = item.ms[dir];
-	       }
+	       } else
+		 spanned = true;
 	     }
 
 	     if (debug)
@@ -407,9 +504,11 @@ WT_DECLARE_WT_MEMBER
 			   + item.ps[1]);
 
 	     // XXX second condition is a hack for WTextEdit
-	     if (item.w.style.display !== 'none'
+	     if ((item.w.style.display !== 'none'
 		 || (WT.hasTag(item.w, 'TEXTAREA') && item.w.wtResize))
+		 && (!item.span || item.span[dir] == 1)) {
 	       allHidden = false;
+	     }
 	   }
 	 }
        }
@@ -431,6 +530,57 @@ WT_DECLARE_WT_MEMBER
        }
      }
 
+     if (spanned) {
+       if (debug)
+	 console.log("(before spanned) "
+		     + id + ': ' + dir + " ps " + preferredSize);
+       for (di = 0; di < dirCount; ++di) {
+	 for (oi = 0; oi < otherCount; ++oi) {
+	   var item = DC.getItem(di, oi);
+
+	   if (item && item.span && item.span[dir] > 1) {
+	     var ps = item.ps[dir], count = 0, stretch = 0;
+
+	     for (si = 0; si < item.span[dir]; ++si) {
+	       var cps = preferredSize[di + si];
+
+	       if (cps != -1) {
+		 ps -= cps;
+		 ++count;
+		 if (DC.config[di + si][STRETCH] > 0)
+		   stretch += DC.config[di + si][STRETCH];
+	       }
+	     }
+
+	     if (ps > 0) {
+	       if (count > 0) {
+		 if (stretch > 0)
+		   count = stretch;
+
+		 for (si = 0; si < item.span[dir]; ++si) {
+		   var cps = preferredSize[di + si];
+		   if (cps != -1) {
+		     var portion;
+		     if (stretch > 0)
+		       portion = DC.config[di + si][STRETCH];
+		     else
+		       portion = 1;
+
+		     if (portion > 0) {
+		       var fract = Math.round(ps / portion);
+		       ps -= fract; count -= portion;
+		       preferredSize[di + si] += fract;
+		     }
+		   }
+		 }
+	       } else
+		 preferredSize[di] = ps;
+	     }
+	   }
+	 }
+       }
+     }
+
      var totalMargin = 0, first = true, rh = false;
      for (di = 0; di < dirCount; ++di) {
        if (minimumSize[di] > -1) {
@@ -464,27 +614,12 @@ WT_DECLARE_WT_MEMBER
 	     totalMargin
 	     ];
 
-     /*
-      * If we are directly in a parent layout, then we want to
-      * mark the corresponding cell as dirty if the TOTAL_PREFERRED_SIZE
-      * has changed.
-      */
-     if (parent) {
-       if (prevMeasures[TOTAL_PREFERRED_SIZE]
-	   != DC.measures[TOTAL_PREFERRED_SIZE]) {
-	 var totalPs = DC.measures[TOTAL_PREFERRED_SIZE];
+     if (layoutDirty ||
+	 (prevMeasures[TOTAL_PREFERRED_SIZE] !=
+	  DC.measures[TOTAL_PREFERRED_SIZE]))
+       self.updateSizeInParent(dir);
 
-	 if (dir == HORIZONTAL)
-	   totalPs = Math.max(totalPs, parentMinPW);
-
-	 totalPs += parentMargin[dir];
-	   
-	 parent.setChildSize(parentItemWidget, dir, totalPs);
-       }
-     }
-
-     /*
-      * If our minimum layout requirements have changed, then we want
+     /* If our minimum layout requirements have changed, then we want
       * to communicate this up using the minimum widths
       *  -- FIXME IE6
       */
@@ -508,14 +643,67 @@ WT_DECLARE_WT_MEMBER
      }
    }
 
-   function finishResize(dir, di, delta) {
+   this.updateSizeInParent = function(dir) {
      /*
-      * fix size of di at current size + delta
+      * If we are directly in a parent layout, then we want to
+      * mark the corresponding cell as dirty if the TOTAL_PREFERRED_SIZE
+      * has changed (or force).
       */
+     if (parent) {
+       var DC = DirConfig[dir],
+           totalPs = DC.measures[TOTAL_PREFERRED_SIZE];
+
+       if (DC.maxSize > 0)
+	 totalPs = Math.min(DC.maxSize, totalPs);
+
+       if (parentWithWtPS) {
+	 var widget = WT.getElement(id);
+
+	 if (!widget)
+	   return;
+
+	 /*
+	  * Go back up and apply wtGetPS() to totalPs
+	  */
+	 var c = widget, p = c.parentNode;
+
+	 for (;;) {
+	   if (p.wtGetPS)
+	     totalPs = p.wtGetPS(p, c, dir, totalPs);
+
+	   totalPs += boxMargin(p, dir);
+
+	   if (p == parentItemWidget)
+	     break;
+
+	   if (p == widget.parentNode && !p.lh && p.offsetHeight > totalPs)
+	     totalPs = p.offsetHeight;
+
+	   c = p;
+	   p = c.parentNode;
+	 }
+       } else
+	 totalPs += parentMargin[dir];
+
+       parent.setChildSize(parentItemWidget, dir, totalPs);
+     }
+   };
+
+   function finishResize(dir, di, delta) {
      var DC = DirConfig[dir];
 
      if (rtl) delta = -delta;
 
+     /*
+      * If di is stretchable and di + 1 isn't, then fix the size of
+      * di+1 (this seems to match with user intuition after around
+      * midnight).
+      */
+     if (DC.config[di][STRETCH] > 0 && DC.config[di + 1][STRETCH] == 0) {
+       ++di;
+       delta = -delta;
+     }
+
      DC.fixedSize[di] = DC.sizes[di] + delta;
 
      APP.layouts2.scheduleAdjust();
@@ -568,40 +756,57 @@ WT_DECLARE_WT_MEMBER
 	 if (cSize === 0) {
 	   if (!DC.initialized) {
 
-	     if (pc !== 'absolute') {
-	       cSize = dir ? container.clientHeight : container.clientWidth;
-
-	       cClientSize = true;
-
-	       if (dir == 0 && cSize == 0 && WT.isIElt9) {
-		 cSize = container.offsetWidth;
-		 cClientSize = false;
-	       }
-
+	     if (dir === HORIZONTAL && (pc === 'absolute' || pc === 'fixed')) {
 	       /*
-		* heuristic to switch to layout-sizes-container mode
+		* On Chrome, somehow clientWidth is not reliable until
+		* we do this little hocus pocus
 		*/
-	       var minSize, ieCSize;
+	       container.style.display = 'none';
+	       cSize = container.clientWidth;
+	       container.style.display = '';
+	     }
 
-	       if ((WT.hasTag(container, "TD") || WT.hasTag(container, "TH"))
-		   && !(WT.isIE && !WT.isIElt9)) {
-		 minSize = 0;
-		 ieCSize = 1;
-	       } else {
-		 if (DC.minSize) // original minSize
-		   minSize = DC.minSize;
-		 else            // set minSize
-		   minSize = measures[TOTAL_MINIMUM_SIZE];
-		 ieCSize = 0;
-	       }
+	     cSize = dir ? container.clientHeight : container.clientWidth;
 
-	       if ((WT.isIElt9 && cSize == ieCSize)
-		   || (cSize == minSize + padding(container, dir))) {
-		 if (debug)
-		   console.log('switching to managed container size '
-			       + dir + ' ' + id);
-		 DC.maxSize = 999999;
-	       }
+	     cClientSize = true;
+
+	     if (dir == 0 && cSize == 0 && WT.isIElt9) {
+	       cSize = container.offsetWidth;
+	       cClientSize = false;
+	     }
+
+	     /*
+	      * heuristic to switch to layout-sizes-container mode
+	      */
+	     var minSize, ieCSize;
+
+	     if ((WT.hasTag(container, "TD") || WT.hasTag(container, "TH"))
+		 && !(WT.isIE && !WT.isIElt9)) {
+	       minSize = 0;
+	       ieCSize = 1;
+	     } else {
+	       if (DC.minSize) // original minSize
+		 minSize = DC.minSize;
+	       else            // set minSize
+		 minSize = measures[TOTAL_MINIMUM_SIZE];
+	       ieCSize = 0;
+	     }
+
+	     if (debug)
+	       console.log('cSize : ' + cSize + ', ieCSize : ' + ieCSize
+			   + ', minSize : ' + minSize + ', padding: '
+			   + padding(container, dir));
+
+	     function epsEqual(a, b) {
+	       return Math.abs(a - b) < 1;
+	     }
+
+	     if ((WT.isIElt9 && epsEqual(cSize, ieCSize))
+		 || (epsEqual(cSize, minSize + padding(container, dir)))) {
+	       if (debug)
+		 console.log('switching to managed container size '
+			     + dir + ' ' + id);
+	       DC.maxSize = 999999;
 	     }
 	   }
 
@@ -614,26 +819,30 @@ WT_DECLARE_WT_MEMBER
 	 cSize = WT.pxself(widget, DC.size);
 	 cPaddedSize = true;
        }
+     } else {
+       if (DC.sizeSet) {
+	 cSize = WT.pxself(container, DC.size);
+	 cPaddedSize = true;
+       }
      }
 
      var otherPadding = 0;
-     if (container
-	 && dir == VERTICAL
-	 && WT.hasTag(container, 'FIELDSET')
-	 && container.children.length == 2) {
-       otherPadding = container.firstChild.offsetHeight;
-     }
+
+     if (container && container.wtGetPS && dir == VERTICAL)
+       otherPadding = container.wtGetPS(container, widget, dir, 0);
 
      var totalPreferredSize = measures[TOTAL_PREFERRED_SIZE];
      if (totalPreferredSize < DC.minSize)
        totalPreferredSize = DC.minSize;
 
-     if (DC.maxSize) {
+     if (DC.maxSize && !DC.sizeSet) {
        // (2) adjust container width/height
        if (totalPreferredSize + otherPadding < DC.maxSize) {
-	 setCss(container, DC.size,
-		(totalPreferredSize
-		 + otherPadding + sizePadding(container, dir)) + 'px');
+	 if (setCss(container, DC.size,
+		    (totalPreferredSize
+		     + otherPadding + sizePadding(container, dir)) + 'px'))
+	   APP.layouts2.remeasure();
+
 	 cSize = totalPreferredSize + otherPadding;
 	 cPaddedSize = true;
 	 noStretch = true;
@@ -648,7 +857,7 @@ WT_DECLARE_WT_MEMBER
      if (dir == VERTICAL && container && container.wtResize) {
        var w = OC.cSize,
 	   h = DC.cSize;
-       container.wtResize(container, w, h);
+       container.wtResize(container, Math.round(w), Math.round(h), true);
      }
 
      cSize -= otherPadding;
@@ -656,7 +865,7 @@ WT_DECLARE_WT_MEMBER
      if (!cPaddedSize) {
        var p = 0;
 
-       if (!DC.initialized) {
+       if (typeof DC.cPadding === 'undefined') {
 	 if (cClientSize)
 	   p = padding(container, dir);
 	 else
@@ -683,15 +892,13 @@ WT_DECLARE_WT_MEMBER
      if (cSize < measures[TOTAL_MINIMUM_SIZE] - otherPadding)
        cSize = measures[TOTAL_MINIMUM_SIZE] - otherPadding;
 
-     $(widget).children("." + OC.handleClass)
-       .css(DC.size,
-	    (cSize - DC.margins[MARGIN_RIGHT] - DC.margins[MARGIN_LEFT])
-	    + 'px');
-
      // (3) compute column/row widths
      var targetSize = [], dirCount = DC.config.length,
        otherCount = OC.config.length;
 
+     for (di = 0; di < dirCount; ++di)
+       DC.stretched[di] = false;
+
      if (cSize >= measures[TOTAL_MINIMUM_SIZE] - otherPadding) {
        // (1) fixed size columns/rows get their fixed width
        // (2) other colums/rows:
@@ -723,7 +930,16 @@ WT_DECLARE_WT_MEMBER
 	 if (measures[MINIMUM_SIZE][di] > -1) {
 
 	   var fs = -1;
-	   if (typeof DC.fixedSize[di] !== "undefined")
+
+	   /*
+	    * If we have a fixedSize (set by resizing) then we should
+	    * take it into account only if the resizer is still visible.
+	    *
+	    * This probably isn't entirely accurate, but neither is resizers
+	    * when hiding/showing widgets ?
+	    */
+	   if (typeof DC.fixedSize[di] !== "undefined"
+	       && (di + 1 == dirCount || measures[MINIMUM_SIZE][di + 1] > -1))
 	     fs = DC.fixedSize[di];
 	   else if ((DC.config[di][RESIZABLE] !== 0)
 		    && (DC.config[di][RESIZABLE][RS_INITIAL_SIZE] >= 0)) {
@@ -752,8 +968,10 @@ WT_DECLARE_WT_MEMBER
 
 	     targetSize[di] = measures[PREFERRED_SIZE][di];
 	   }
-	 } else
+	 } else {
 	   stretch[di] = HIDDEN;
+	   targetSize[di] = -1;
+	 }
        }
 
        if (totalStretch == 0) {
@@ -782,8 +1000,10 @@ WT_DECLARE_WT_MEMBER
 	     var factor = toDistribute / totalStretch;
 
 	     for (di = 0; di < dirCount; ++di) {
-	       if (stretch[di] > 0)
+	       if (stretch[di] > 0) {
 		 targetSize[di] += Math.round(stretch[di] * factor);
+		 DC.stretched[di] = true;
+	       }
 	     }
 	   }
 	 } else {
@@ -849,7 +1069,8 @@ WT_DECLARE_WT_MEMBER
        console.log(" -> targetSize: " + targetSize);
 
      // (4) set widths/heights of cells
-     var left = 0, first = true, resizeHandle = false, oi, di;
+     var left = DC.margins[MARGIN_LEFT], 
+       first = true, resizeHandle = false, oi, di;
 
      for (di = 0; di < dirCount; ++di) {
        if (targetSize[di] > -1) {
@@ -864,6 +1085,9 @@ WT_DECLARE_WT_MEMBER
 	     handle.style.position='absolute';
 	     handle.style[OC.left] = OC.margins[MARGIN_LEFT] + 'px';
 	     handle.style[DC.size] = DC.margins[SPACING] + 'px';
+	     if (OC.cSize)
+	       handle.style[OC.size] = (OC.cSize - OC.margins[MARGIN_RIGHT] -
+					OC.margins[MARGIN_LEFT]) + 'px';
 	     handle.className = DC.handleClass;
 	     widget.insertBefore(handle, widget.firstChild);
 
@@ -880,123 +1104,134 @@ WT_DECLARE_WT_MEMBER
 
 	 resizeHandle = DC.config[di][RESIZABLE] !== 0;
 
-	 if (first) {
-	   left += DC.margins[MARGIN_LEFT];
+	 if (first)
 	   first = false;
-	 } else
+	 else
 	   left += DC.margins[SPACING];
+       }
 
-	 for (oi = 0; oi < otherCount; ++oi) {
-	   var item = DC.getItem(di, oi);
-	   if (item && item.w) {
-	     var w = item.w;
+       for (oi = 0; oi < otherCount; ++oi) {
+	 var item = DC.getItem(di, oi);
+	 if (item && item.w) {
+	   var w = item.w;
 
-	     var ts = targetSize[di];
-	     if (item.span) {
-	       var si;
+	   var ts = Math.max(targetSize[di], 0);
+	   if (item.span) {
+	     var si;
 
-	       var rs = resizeHandle;
-	       for (si = 1; si < item.span[dir]; ++si) {
-		 if (rs)
-		   ts += RESIZE_HANDLE_MARGIN * 2;
-		 rs = DC.config[di + rs][RESIZABLE] !== 0;
+	     var rs = resizeHandle;
+	     for (si = 1; si < item.span[dir]; ++si) {
+	       if (di + si >= targetSize.length)
+		 break;
+
+	       if (rs)
+		 ts += RESIZE_HANDLE_MARGIN * 2;
+
+	       rs = DC.config[di + si][RESIZABLE] !== 0;
+
+	       if (targetSize[di + si - 1] > -1 && targetSize[di + si] > -1)
 		 ts += DC.margins[SPACING];
-		 ts += targetSize[di + si];
-	       }
-	     }
-
-	     var off;
-
-	     w.style.visibility = '';
-
-	     var alignment = (item.align >> DC.alignBits) & 0xF;
-	     var ps = item.ps[dir];
-
-	     if (ts < ps)
-	       alignment = 0;
-
-	     if (!alignment) {
-	       var m = margin(item.w, dir);
-
-	       /*
-		* Chrome:
-		*  - some elements loose their margin as soon as you give
-		*    them a size, we thus need to correct for that, or
-		*    confirm their initial margin
-		*/
-	       var tsm = ts;
-	       if (WT.isIElt9 ||
-		   (!WT.hasTag(w, 'BUTTON') && !WT.hasTag(w, 'INPUT') &&
-		    !WT.hasTag(w, 'SELECT') && !WT.hasTag(w, 'TEXTAREA')))
-		 tsm = Math.max(0, tsm - m);
-
-	       /*
-		* IE: a button expands to parent container width ?? WTF ?
-		*/
-	       var setSize = false;
-	       if (WT.isIE && WT.hasTag(w, 'BUTTON'))
-		 setSize = true;
-
-	       if (setSize || ts != ps || item.layout) {
-		 if (setCss(w, DC.size, tsm + 'px'))
-		   setItemDirty(item);
-		 item.set[dir] = true;
-	       } else {
-		 if (!item.fs[dir]) {
-		   if (setCss(w, DC.size, ''))
-		     setItemDirty(item);
-		   item.set[dir] = false;
-		 } else if (dir == HORIZONTAL)
-		   setCss(w, DC.size, item.fs[dir] + 'px');
-	       }
-
-	       off = left;
-	       item.size[dir] = tsm;
-	       item.psize[dir] = ts;
-	     } else {
-	       switch (alignment) {
-	       case ALIGN_LEFT: off = left; break;
-	       case ALIGN_CENTER: off = left + (ts - ps)/2; break;
-	       case ALIGN_RIGHT: off = left + (ts - ps); break;
-	       }
-
-	       if (item.layout) {
-		 if (setCss(w, DC.size, ps + 'px'))
-		   setItemDirty(item);
-		 item.set[dir] = true;
-	       } else if (ts >= ps && item.set[dir]) {
-		 if (setCss(w, DC.size, ps + 'px'))
-		   setItemDirty(item);
-		 item.set[dir] = false;
-	       }
-
-	       item.size[dir] = ps;
-	       item.psize[dir] = ps;
-	     }
-
-	     if (!progressive)
-	       setCss(w, DC.left, off + 'px');
-	     else
-	       if (thisResized) {
-		 setCss(w, DC.left, (RESIZE_HANDLE_MARGIN * 2) + 'px');
-		 var pc = WT.css(w, 'position');
-		 if (pc !== 'absolute')
-		   w.style.position = 'relative';
-	       } else
-		 setCss(w, DC.left, '0px');
-
-	     if (dir == VERTICAL) {
-	       if (w.wtResize)
-		 w.wtResize(w, item.size[HORIZONTAL], item.size[VERTICAL]);
-
-	       item.dirty = false;
+	       ts += targetSize[di + si];
 	     }
 	   }
-	 }
+	   
+	   var off;
 
-	 left += targetSize[di];
+	   setCss(w, 'visibility', '');
+
+	   var alignment = (item.align >> DC.alignBits) & 0xF;
+	   var ps = item.ps[dir];
+
+	   if (ts < ps)
+	     alignment = 0;
+
+	   if (!alignment) {
+	     var m = margin(item.w, dir);
+
+	     var tsm = Math.max(0, ts - m);
+
+	     if (!WT.isIE && WT.hasTag(w, 'TEXTAREA'))
+	       tsm = ts;
+
+	     /*
+	      * IE: a button expands to parent container width ?? WTF ?
+	      */
+	     var setSize = false;
+	     if (WT.isIE && WT.hasTag(w, 'BUTTON'))
+	       setSize = true;
+
+	     if (setSize || ts != ps || item.layout) {
+	       if (setCss(w, DC.size, tsm + 'px'))
+		 setItemDirty(item);
+	       item.set[dir] = true;
+	     } else {
+	       if (!item.fs[dir]) {
+		 if (setCss(w, DC.size, ''))
+		   setItemDirty(item);
+		 item.set[dir] = false;
+	       } else if (dir == HORIZONTAL)
+		 setCss(w, DC.size, item.fs[dir] + 'px');
+	     }
+
+	     off = left;
+	     item.size[dir] = tsm;
+	     item.psize[dir] = ts;
+	   } else {
+	     switch (alignment) {
+	     case ALIGN_LEFT: off = left; break;
+	     case ALIGN_CENTER: off = left + (ts - ps)/2; break;
+	     case ALIGN_RIGHT: off = left + (ts - ps); break;
+	     }
+
+	     ps -= margin(item.w, dir);
+
+	     if (item.layout) {
+	       if (setCss(w, DC.size, ps + 'px'))
+		 setItemDirty(item);
+	       item.set[dir] = true;
+	     } else if (ts >= ps && item.set[dir]) {
+	       if (setCss(w, DC.size, ps + 'px'))
+		 setItemDirty(item);
+	       item.set[dir] = false;
+	     }
+
+	     item.size[dir] = ps;
+	     item.psize[dir] = ps;
+	   }
+
+	   if (!progressive)
+	     setCss(w, DC.left, off + 'px');
+	   else
+	     if (thisResized) {
+	       setCss(w, DC.left, (RESIZE_HANDLE_MARGIN * 2) + 'px');
+	       var pc = WT.css(w, 'position');
+	       if (pc !== 'absolute')
+		 w.style.position = 'relative';
+	     } else
+	       setCss(w, DC.left, '0px');
+
+	   if (dir == VERTICAL) {
+	     if (w.wtResize)
+	       w.wtResize(w,
+			  item.set[HORIZONTAL]
+			  ? Math.round(item.size[HORIZONTAL]) : -1,
+			  item.set[VERTICAL]
+			  ? Math.round(item.size[VERTICAL]) : -1,
+			  true);
+
+	     item.dirty = false;
+	   }
+	 }
        }
+
+       if (targetSize[di] > -1)
+	 left += targetSize[di];
      }
+
+     $(widget).children("." + OC.handleClass)
+       .css(DC.size,
+	    (cSize - DC.margins[MARGIN_RIGHT] - DC.margins[MARGIN_LEFT])
+	    + 'px');
    }
 
    this.setConfig = function(conf) {
@@ -1009,15 +1244,20 @@ WT_DECLARE_WT_MEMBER
      DirConfig[0].config = config.cols;
      DirConfig[1].config = config.rows;
 
+     DirConfig[0].stretched = [];
+     DirConfig[1].stretched = [];
+
      var i, il, childLayouts = {};
      for (i = 0, il = oldConfig.items.length; i < il; ++i) {
        var item = oldConfig.items[i];
 
        if (item) {
-	 if (item.set[HORIZONTAL])
-	   setCss(item.w, DirConfig[HORIZONTAL].size, '');
-	 if (item.set[VERTICAL])
-	   setCss(item.w, DirConfig[VERTICAL].size, '');
+	 if (item.set) {
+	   if (item.set[HORIZONTAL])
+	     setCss(item.w, DirConfig[HORIZONTAL].size, '');
+	   if (item.set[VERTICAL])
+	     setCss(item.w, DirConfig[VERTICAL].size, '');
+	 }
 
 	 if (item.layout) {
 	   self.setChildSize(item.w, HORIZONTAL, item.ps[HORIZONTAL]);
@@ -1027,6 +1267,8 @@ WT_DECLARE_WT_MEMBER
      }
 
      layoutDirty = true;
+
+     APP.layouts2.scheduleAdjust();
    };
 
    this.getId = function() {
@@ -1035,10 +1277,28 @@ WT_DECLARE_WT_MEMBER
 
    this.setItemsDirty = function(items) {
      var i, il;
+
      var colCount = DirConfig[HORIZONTAL].config.length;
      for (i = 0, il = items.length; i < il; ++i) {
        var row = items[i][0], col = items[i][1];
-       config.items[row * colCount + col].dirty = true;
+
+       var item = config.items[row * colCount + col];
+
+       item.dirty = true;
+
+       /*
+	* When the item contains a layout, a change may also impact this:
+	* it could become a different layout (e.g. WStackedWidget) or
+	* a hidden layout (e.g. WPanel). Assume in general that it could
+	* be the case that we need to reset the layout-based size, but don't
+	* do it yet since it causes flicker when nothing drastic changed.
+	*/
+       if (item.layout) {
+	 item.layout = false;
+	 item.wasLayout = true;
+
+	 APP.layouts2.setChildLayoutsDirty(self, item.w);
+       }
      }
 
      itemDirty = true;
@@ -1049,20 +1309,28 @@ WT_DECLARE_WT_MEMBER
    };
 
    this.setChildSize = function(widget, dir, preferredSize) {
-     var i, il;
+     var colCount = DirConfig[HORIZONTAL].config.length,
+         DC = DirConfig[dir],
+         i, il;
+
      for (i = 0, il = config.items.length; i < il; ++i) {
        var item = config.items[i];
        if (item && item.id == widget.id) {
-	 if (!item.ps)
-	   item.ps = [];
+	 var di = (dir === HORIZONTAL ? i % colCount : i / colCount);
+	 var alignment = (item.align >> DC.alignBits) & 0xF;
+
+	 if (alignment || !DC.stretched[di]) {
+	   if (!item.ps)
+	     item.ps = [];
+	   item.ps[dir] = preferredSize;
+	 }
 
-	 item.ps[dir] = preferredSize;
 	 item.layout = true;
+
+	 setItemDirty(item);
 	 break;
        }
      }
-
-     itemDirty = true;
    };
 
    this.measure = function(dir) {
@@ -1085,8 +1353,9 @@ WT_DECLARE_WT_MEMBER
 	 parentMargin[VERTICAL] = boxMargin(parentItemWidget, VERTICAL);
        } else {
 	 /*
-	  * While we are a single child in a parent, we can go further
-	  * up looking for an ancestor layout
+	  * While we are a single child in a parent, or if we have a
+	  * wtGetPS() function on that parent, we can go further up
+	  * looking for an ancestor layout
 	  */
 	 var c = widget, p = c.parentNode;
 
@@ -1095,16 +1364,8 @@ WT_DECLARE_WT_MEMBER
 	   parentMargin[HORIZONTAL] += boxMargin(p, HORIZONTAL);
 	   parentMargin[VERTICAL] += boxMargin(p, VERTICAL);
 
-	   if (p.childNodes.length > 1) {
-	     var i, il;
-	     for (i = 0, il = p.childNodes.length; i < il; ++i) {
-	       if (p.childNodes[i] != c) {
-		 parentMargin[VERTICAL] += p.childNodes[i].offsetHeight;
-		 parentMinPW
-		   = Math.max(p.childNodes[i].offsetWidth, parentMinPW);
-	       }
-	     }
-	   }
+	   if (p.wtGetPS)
+	     parentWithWtPS = true;
 
 	   var l = jQuery.data(p.parentNode, 'layout');
 	   if (l) {
@@ -1115,9 +1376,14 @@ WT_DECLARE_WT_MEMBER
 
 	   c = p;
 	   p = c.parentNode;
-	   if (p.childNodes.length != 1)
+
+	   if (p.childNodes.length != 1 && !p.wtGetPS)
 	     break;
 	 }
+
+	 var container = widget.parentNode;
+	 for (var i = 0; i < 2; ++i)
+	   DirConfig[i].sizeSet = WT.pxself(container, DirConfig[i].size) != 0;
        }
      }
 
@@ -1130,6 +1396,11 @@ WT_DECLARE_WT_MEMBER
        itemDirty = layoutDirty = false;
    };
 
+   this.setMaxSize = function(width, height) {
+     DirConfig[HORIZONTAL].maxSize = width;
+     DirConfig[VERTICAL].maxSize = height;
+   };
+
    this.apply = function(dir) {
      var widget = WT.getElement(id);
 
@@ -1153,6 +1424,8 @@ WT_DECLARE_WT_MEMBER
      else
        return false;
    };
+
+   this.WT = WT;
  });
 
 WT_DECLARE_WT_MEMBER
@@ -1181,6 +1454,22 @@ WT_DECLARE_APP_MEMBER
 	layout.setDirty();
     };
 
+    this.setChildLayoutsDirty = function(layout, itemWidget) {
+      var i, il;
+
+      for (i=0, il = layout.descendants.length; i < il; ++i) {
+	var ll = layout.descendants[i];
+
+	if (itemWidget) {
+	  var lw = layout.WT.getElement(ll.getId());
+	  if (lw && !layout.WT.contains(itemWidget, lw))
+	    continue;
+	}
+
+	ll.setDirty();
+      }
+    };
+
     this.add = function(layout) {
       function addIn(layouts, layout) {
 	var i, il;
@@ -1188,7 +1477,15 @@ WT_DECLARE_APP_MEMBER
 	for (i=0, il = layouts.length; i < il; ++i) {
 	  var ll = layouts[i];
 
-	  if (ll.contains(layout)) {
+	  if (ll.getId() == layout.getId()) {
+	    /*
+	     * Is being re-added (nested layout of item removed + added
+	     * in same event)
+	     */
+	    layouts[i] = layout;
+	    layout.descendants = ll.descendants;
+	    return;
+	  } else if (ll.contains(layout)) {
 	    addIn(ll.descendants, layout);
 	    return;
 	  } else if (layout.contains(ll)) {
@@ -1202,11 +1499,16 @@ WT_DECLARE_APP_MEMBER
       }
 
       addIn(topLevelLayouts, layout);
+
+      self.scheduleAdjust();
     };
 
-    var adjustScheduled = false;
+    var adjustScheduled = false, needRemeasure = false;
+
+    this.scheduleAdjust = function(forceMeasureVertical) {
+      if (forceMeasureVertical)
+	measureVertical = true;
 
-    this.scheduleAdjust = function() {
       if (adjustScheduled)
 	return;
 
@@ -1221,6 +1523,8 @@ WT_DECLARE_APP_MEMBER
 	if (layout)
 	  layout.setItemsDirty(items);
 
+	self.scheduleAdjust();
+
 	return;
       }
 
@@ -1230,6 +1534,7 @@ WT_DECLARE_APP_MEMBER
 	return;
 
       adjusting = true;
+      needRemeasure = false;
 
       function measure(layouts, dir) {
 	var i, il;
@@ -1268,7 +1573,15 @@ WT_DECLARE_APP_MEMBER
       measure(topLevelLayouts, VERTICAL);
       apply(topLevelLayouts, VERTICAL);
 
+      if (needRemeasure) {
+	measure(topLevelLayouts, HORIZONTAL);
+	apply(topLevelLayouts, HORIZONTAL);
+	measure(topLevelLayouts, VERTICAL);
+	apply(topLevelLayouts, VERTICAL);
+      }
+
       adjusting = false;
+      needRemeasure = false;
       measureVertical = false;
     };
 
@@ -1280,10 +1593,18 @@ WT_DECLARE_APP_MEMBER
       return;
     };
 
-    window.onresize = function() {
-      measureVertical = true;
-
-      self.scheduleAdjust();
+    this.remeasure = function() {
+      needRemeasure = true;
     };
+
+    window.onresize = function() {
+      self.scheduleAdjust(true);
+    };
+
+    window.onshow = function() {
+      measureVertical = true;
+      self.adjust();
+    };
+
   }) ()
 );
