@@ -8,11 +8,10 @@
 
 WT_DECLARE_WT_MEMBER
 (1, JavaScriptConstructor, "WDialog",
- function(APP, el, centerX, centerY) {
+ function(APP, el, titlebar, centerX, centerY) {
    jQuery.data(el, 'obj', this);
 
    var self = this;
-   var titlebar = $(el).find(".titlebar").first().get(0);
    var layoutContainer = $(el).find(".dialog-layout").get(0);
    var WT = APP.WT;
    var dsx, dsy;
@@ -74,18 +73,20 @@ WT_DECLARE_WT_MEMBER
 	     + (WT.isIE6 ? document.documentElement.scrollTop : 0)) + 'px';
 	 el.style.marginTop = '0px';
        }
-     }
 
-     if (el.style.position != '')
-       el.style.visibility = 'visible';
+       if (el.style.position != '') {
+	 el.style.visibility = 'visible';
+       }
+     }
    };
 
    function layoutResize(ignored, w, h) {
      if (el.style.position == '') {
        el.style.position = WT.isIE6 ? 'absolute' : 'fixed';
-       el.style.visibility = 'visible';
      }
 
+     el.style.visibility = 'visible';
+
      el.style.height = Math.max(0, h) + 'px';
      el.style.width = Math.max(0, w) + 'px';
 
@@ -109,6 +110,9 @@ WT_DECLARE_WT_MEMBER
      centerX = centerY = false;
      wtResize(el, w, h);
 
+     var layout = jQuery.data(layoutContainer.firstChild, 'layout');
+     layout.setMaxSize(0, 0);
+
      APP.layouts2.scheduleAdjust();
    };
 
