@@ -12,15 +12,22 @@
 
 WT_DECLARE_WT_MEMBER
 (1, JavaScriptConstructor, "WDoubleValidator",
-   function(mandatory, bottom, top, blankError,
+   function(mandatory, bottom, top, decimalPoint, groupSeparator, blankError,
 	    NaNError, tooSmallError, tooLargeError) {
      this.validate = function(text) {
+       text = String(text);
+
        if (text.length == 0)
 	 if (mandatory)
 	   return { valid: false, message: blankError };
 	 else
 	   return { valid: true };
 
+       if (groupSeparator != '')
+	 text = text.replace(groupSeparator, '');
+       if (decimalPoint != '.')
+	 text = text.replace(decimalPoint, '.');
+
        var n = Number(text);
 
        if (isNaN(n))
