@@ -17,6 +17,8 @@ WT_DECLARE_WT_MEMBER
    var self = this;
    var WT = APP.WT;
 
+   var sizeSet = false;
+
    /** @const */ var EnsureVisible = 0;
    /** @const */ var PositionAtTop = 1;
    /** @const */ var PositionAtBottom = 2;
@@ -28,38 +30,38 @@ WT_DECLARE_WT_MEMBER
 
      var t = WT.target(event);
 
-     while (t) {
-       if (t.className.indexOf('c1 rh') == 0) {
+     while (t && t != el) {
+       if (WT.hasTag(t, 'LI')) {
 	 if (columnId == -1)
            columnId = 0;
+	 nodeId = t.id;
+
+	 break;
        } else if (t.className.indexOf('Wt-tv-c') == 0) {
 	 if (t.className.indexOf('Wt-tv-c') == 0)
-           columnId = t.className.split(' ')[0].substring(7) * 1;
+	   columnId = t.className.split(' ')[0].substring(7) * 1;
 	 else if (columnId == -1)
-	 columnId = 0;
+	   columnId = 0;
 	 if (t.getAttribute('drop') === 'true')
 	   drop = true;
 	 ele = t;
-       } else if (t.className == 'Wt-tv-node') {
-	 nodeId = t.id;
-	 break;
        }
-       if (t.className === 'Wt-selected')
+
+       if ($(t).hasClass('Wt-selected'))
 	 selected = true;
+
        t = t.parentNode;
-       if (WT.hasTag(t, 'BODY'))
-	 break;
      }
 
      return { columnId: columnId, nodeId: nodeId, selected: selected,
 	      drop: drop, el: ele };
-   };
+   }
 
    this.click = function(obj, event) {
      var item = getItem(event);
      if (item.columnId != -1) {
        APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
-		item.nodeId, item.columnId, 'clicked', '', '');
+		item.nodeId + ':' + item.columnId, 'clicked', '', '');
      }
    };
 
@@ -67,7 +69,7 @@ WT_DECLARE_WT_MEMBER
      var item = getItem(event);
      if (item.columnId != -1) {
        APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
-		item.nodeId, item.columnId, 'dblclicked', '', '');
+		item.nodeId + ':' + item.columnId, 'dblclicked', '', '');
      }
    };
 
@@ -76,7 +78,7 @@ WT_DECLARE_WT_MEMBER
      var item = getItem(event);
      if (item.columnId != -1) {
        APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
-		item.nodeId, item.columnId, 'mousedown', '', '');
+		item.nodeId + ':' + item.columnId, 'mousedown', '', '');
        if (el.getAttribute('drag') === 'true' && item.selected)
          APP._p_.dragStart(el, event);
      }
@@ -86,10 +88,30 @@ WT_DECLARE_WT_MEMBER
      var item = getItem(event);
      if (item.columnId != -1) {
        APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
-		item.nodeId, item.columnId, 'mouseup', '', '');
+		item.nodeId + ':' + item.columnId, 'mouseup', '', '');
      }
    };
 
+   this.rootClick = function(obj, event) {
+     APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
+	      '', 'clicked', '', '');
+   };
+
+   this.rootDblclick = function(obj, event) {
+     APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
+	      '', 'dblclicked', '', '');
+   };
+
+   this.rootMouseDown = function(obj, event) {
+     APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
+	      '', 'mousedown', '', '');
+   };
+
+   this.rootMouseUp = function(obj, event) {
+     APP.emit(el, { name: 'itemEvent', eventObject: obj, event: event },
+	      '', 'mouseup', '', '');
+   };
+
    this.resizeHandleMDown = function(obj, event) {
      var parent = obj.parentNode,
          c = parent.className.split(' ')[0];
@@ -135,10 +157,10 @@ WT_DECLARE_WT_MEMBER
    var adjustScheduled = false;
 
    function doAdjustColumns() {
-     if (!adjustScheduled)
+     if (!adjustScheduled || !sizeSet)
        return;
 
-     if (WT.isHidden(el) || el.offsetWidth < 20)
+     if (WT.isHidden(el))
        return;
 
      adjustScheduled = false;
@@ -180,7 +202,8 @@ WT_DECLARE_WT_MEMBER
 
        if (!c0r.style.width) {
 	 // first resize and c0 width not set
-	 var c0rw = el.scrollWidth - hc.offsetWidth - 8;
+	 // 9 = 2px (widget border) - 7px column padding - 22px (scrollbar room)
+	 var c0rw = el.scrollWidth - hc.offsetWidth - 9 - 15;
 	 if (c0rw > 0)
 	   c0r.style.width = c0rw + 'px';
        } else
@@ -201,6 +224,8 @@ WT_DECLARE_WT_MEMBER
        var r = WT.getCssRule('#' + el.id + ' .Wt-tv-rowc');
        r.style.width = allw_1 + 'px';
 
+       APP.layouts2.adjust();
+
        if (WT.isIE) {
 	 setTimeout(function() {
 	     $(el).find('.Wt-tv-rowc')
@@ -238,7 +263,7 @@ WT_DECLARE_WT_MEMBER
      if (!item.selected && item.drop && item.columnId != -1) {
        if (action=='drop') {
 	 APP.emit(el, { name: 'itemEvent', eventObject: object, event: event },
-		  item.nodeId, item.columnId, 'drop', sourceId, mimeType);
+		  item.nodeId + ':' + item.columnId, 'drop', sourceId, mimeType);
        } else {
          object.className = 'Wt-valid-drop';
          dropEl = item.el;
@@ -261,6 +286,8 @@ WT_DECLARE_WT_MEMBER
    */
 
   this.wtResize = function() {
+      sizeSet = true;
+
       doAdjustColumns();
 
       var $el=$(el),
@@ -278,6 +305,10 @@ WT_DECLARE_WT_MEMBER
       var scrollwidth = contentsContainer.offsetWidth
         - contentsContainer.clientWidth;
 
+      // IE cannot accurately estimate scrollwidth from time to time ?
+      if (scrollwidth > 50)
+	scrollwidth = 0;
+
       if (contentsContainer.clientWidth > 0)
 	tw -= scrollwidth;
 
@@ -312,8 +343,11 @@ WT_DECLARE_WT_MEMBER
 	contentsContainer.style.width = (tw + scrollwidth) + 'px';
 
 	// IE moves the scrollbar left in rtl mode.
-	if (!WT.isIE)
+	var rtl = $(document.body).hasClass('Wt-rtl');
+	if (!rtl || !WT.isIElt9) {
 	  headerContainer.style.marginRight = scrollwidth + 'px';
+	  $('#' + el.id + ' .Wt-scroll').css('marginRight', scrollwidth + 'px');
+	}
 
 	if (c0w != null) {
 	  var w = tw - c0w - (WT.isIE6 ? 10 : 7);
