@@ -11,6 +11,7 @@
 #include <map>
 #include <iostream>
 #include <vector>
+#include <boost/cstdint.hpp>
 
 #include <Wt/WDllDefs.h>
 
@@ -25,23 +26,25 @@ class WebRequest;
 class CgiParser
 {
 public:
+  enum ReadOption { ReadDefault, ReadHeadersOnly, ReadBodyAnyway };
+
   static void init();
 
-  CgiParser(int maxPostData);
+  CgiParser(::int64_t maxPostData);
 
   /*
    * Reads in GET or POST data, converts it to unescaped text, and
    * creates Entry for each parameter entry. The request is annotated
    * with the parse results.
    */
-  void parse(WebRequest& request);
+  void parse(WebRequest& request, ReadOption option);
 
 private:
-  void readMultipartData(WebRequest& request, const std::string type, int len);
+  void readMultipartData(WebRequest& request, const std::string type,
+			 ::int64_t len);
   bool parseBody(WebRequest& request, const std::string boundary);
   bool parseHead(WebRequest& request);
-  unsigned maxPostData_;
-  int left_;
+  ::int64_t maxPostData_, left_;
   std::ostream *spoolStream_;
   WebRequest *request_;
 
@@ -54,8 +57,8 @@ private:
   void windBuffer(int offset);
   int index(const std::string search);
 
-  static const int BUFSIZE = 8192;
-  static const int MAXBOUND = 100;
+  enum {BUFSIZE = 8192};
+  enum {MAXBOUND = 100};
 
   int buflen_;
   static char buf_[BUFSIZE + MAXBOUND];
