@@ -112,8 +112,10 @@ public:
 
 #ifndef WT_TARGET_JAVA
   void addEntryPoint(const EntryPoint& entryPoint);
+  void removeEntryPoint(const std::string& path);
   void setDefaultEntryPoint(const std::string& path);
   const EntryPointList& entryPoints() const { return entryPoints_; }
+  void setNumThreads(int threads);
 #endif // WT_TARGET_JAVA
 
   SessionPolicy sessionPolicy() const;
@@ -127,6 +129,7 @@ public:
   int sessionTimeout() const;
   int bootstrapTimeout() const;
   int indicatorTimeout() const;
+  int doubleClickTimeout() const;
   int serverPushTimeout() const;
   std::string valgrindPath() const;
   ErrorReporting errorReporting() const;
@@ -160,6 +163,7 @@ public:
 
   bool agentIsBot(const std::string& agent) const;
   bool agentSupportsAjax(const std::string& agent) const;
+  std::string uaCompatible() const;
 
   // Things which are overridden by the connector
   void setSessionTimeout(int sessionTimeout);
@@ -182,6 +186,7 @@ private:
   std::string applicationPath_;
   std::string appRoot_;
   std::string configurationFile_;
+  std::string uaCompatible_;
 
 #ifndef WT_TARGET_JAVA
   EntryPointList entryPoints_;
@@ -198,6 +203,7 @@ private:
   int             sessionTimeout_;
   int             bootstrapTimeout_;
   int		  indicatorTimeout_;
+  int             doubleClickTimeout_;
   int             serverPushTimeout_;
   std::string     valgrindPath_;
   ErrorReporting  errorReporting_;
