@@ -20,6 +20,7 @@ public:
 
   EscapeOStream();
   EscapeOStream(std::ostream& sink);
+  EscapeOStream(WStringStream& sink);
   EscapeOStream(EscapeOStream& other);
 
   void pushEscape(RuleSet rules);
@@ -30,11 +31,27 @@ public:
 #endif // WT_TARGET_JAVA
 
   void append(const std::string& s, const EscapeOStream& rules);
+  void append(const char *s, std::size_t len);
+
+#ifndef WT_TARGET_JAVA
+  /*
+   * Should not be implemented but is needed to support the specialization
+   * for string literals !
+   */
+  template <typename T>
+    inline EscapeOStream& operator<< (T t);
+
+  template <std::size_t N>
+    EscapeOStream& operator<< (const char (&s)[N]) {
+    append(s, N-1); return *this; 
+  }
+#endif // WT_TARGET_JAVA
 
   EscapeOStream& operator<< (char);
-  EscapeOStream& operator<< (const char *s);
+  EscapeOStream& operator<< (char *s);
   EscapeOStream& operator<< (const std::string& s);
   EscapeOStream& operator<< (int);
+  EscapeOStream& operator<< (bool);
   EscapeOStream& operator<< (const EscapeOStream& other);
 
   const char *c_str(); // for default constructor, can return 0
@@ -44,7 +61,8 @@ public:
   void clear();
 
 private:
-  WStringStream stream_;
+  WStringStream own_stream_;
+  WStringStream& stream_;
 
   struct Entry {
     char c;
