@@ -7,18 +7,17 @@
 #ifndef SOUNDMANAGER_H_
 #define SOUNDMANAGER_H_
 
-#include <Wt/WObject>
+#include <Wt/WMediaPlayer>
 
 namespace Wt {
 
 class WApplication;
 class WSound;
 
-class SoundManager : public WObject
+class SoundManager : public WMediaPlayer
 {
 public:
-  SoundManager(WApplication *app);
-  ~SoundManager();
+  SoundManager(WContainerWidget *parent = 0);
 
   void add(WSound *sound);
   void remove(WSound *sound);
@@ -27,11 +26,6 @@ public:
   void stop(WSound *sound);
 
   bool isFinished(WSound *sound) const;
-
-  int loopsRemaining(WSound *sound) const;
-
-private:
-  WApplication *wApp_;
 };
 
 }
