@@ -24,6 +24,7 @@
 
 #include "TimeUtil.h"
 #include "WebRenderer.h"
+#include "WebRequest.h"
 
 #include "Wt/WApplication"
 #include "Wt/WEnvironment"
@@ -62,6 +63,10 @@ public:
 	     const WebRequest *request, WEnvironment *env = 0);
   ~WebSession();
 
+#ifdef WT_TARGET_JAVA
+  void destruct();
+#endif // WT_TARGET_JAVA
+
   static WebSession *instance();
 
   bool attachThreadToLockedHandler();
@@ -93,7 +98,6 @@ public:
   void pushUpdates();
 
   void doRecursiveEventLoop();
-  bool bootStyleResponse() const { return !noBootStyleResponse_; }
 
   void deferRendering();
   void resumeRendering();
@@ -127,6 +131,7 @@ public:
   const std::string& deploymentPath() const { return deploymentPath_; }
 
   bool hasSessionIdInUrl() const { return sessionIdInUrl_; }
+  void setSessionIdInUrl(bool value) { sessionIdInUrl_ = value; }
   bool useUglyInternalPaths() const;
   void setPagePathInfo(const std::string& path);
   std::string pagePathInfo() const { return pagePathInfo_; }
@@ -192,6 +197,7 @@ public:
     std::vector<unsigned int> signalOrder;
 
 #ifdef WT_THREADED
+    boost::thread::id lockOwner() const { return lockOwner_; }
     boost::mutex::scoped_lock& lock() { return lock_; }
 #endif
 
@@ -203,6 +209,7 @@ public:
 
 #ifdef WT_THREADED
     boost::mutex::scoped_lock lock_;
+    boost::thread::id lockOwner_;
 
     Handler(const Handler&);
 #endif // WT_THREADED
@@ -235,21 +242,10 @@ public:
 
   void generateNewSessionId();
 
-#if defined(WT_THREADED)
-  /*
-   * SyncLocks
-   */
-  struct SyncLocks {
-    boost::mutex state_;
-    boost::condition unlock_;
-    int lastId_;
-    int lockedId_;
-  } syncLocks_;
-#endif // WT_THREADED
-
 private:
   void handleWebSocketRequest(Handler& handler);
-  static void handleWebSocketMessage(boost::weak_ptr<WebSession> session);
+  static void handleWebSocketMessage(boost::weak_ptr<WebSession> session,
+				     WebRequest::ReadEvent event);
   static void webSocketReady(boost::weak_ptr<WebSession> session);
 
   void checkTimers();
@@ -277,8 +273,9 @@ private:
   std::string applicationUrl_, deploymentPath_;
   std::string redirect_;
   std::string pagePathInfo_;
+  std::string pongMessage_;
   WebResponse *asyncResponse_, *bootStyleResponse_;
-  bool canWriteAsyncResponse_, noBootStyleResponse_;
+  bool canWriteAsyncResponse_;
   int pollRequestsIgnored_;
   bool progressiveBoot_;
 
@@ -338,10 +335,11 @@ private:
 			       const std::string& se) const;
 
   void init(const WebRequest& request);
-  bool start();
+  bool start(WebResponse *response);
 
   std::string sessionQuery() const;
   void flushBootStyleResponse();
+  void changeInternalPath(const std::string& path, WebResponse *response);
 
   friend class WebSocketMessage;
   friend class WebRenderer;
