@@ -16,7 +16,9 @@ window.WT_DECLARE_WT_MEMBER = function(i, type, name, fn)
     _$_WT_CLASS_$_[name.substr(0, proto)]
       .prototype[name.substr(proto + '.prototype.'.length)] = fn;
   } else if (type == JavaScriptFunction) {
-    _$_WT_CLASS_$_[name] = function() { fn.apply(_$_WT_CLASS_$_, arguments); };
+    _$_WT_CLASS_$_[name] = function() {
+	return fn.apply(_$_WT_CLASS_$_, arguments);
+    };
   } else {
     _$_WT_CLASS_$_[name] = fn;
   }
@@ -30,7 +32,7 @@ window.WT_DECLARE_APP_MEMBER = function(i, type, name, fn)
     app[name.substr(0, proto)]
       .prototype[name.substr(proto + '.prototype.'.length)] = fn;
   } else if (type == JavaScriptFunction) {
-    app[name] = function() { fn.apply(app, arguments); };
+    app[name] = function() { return fn.apply(app, arguments); };
   } else {
     app[name] = fn;
   }
@@ -141,6 +143,7 @@ var agent = navigator.userAgent.toLowerCase();
 
 this.isIE = ie !== undefined;
 this.isIE6 = ie === 6;
+this.isIE8 = ie === 8;
 this.isIElt9 = ie < 9;
 this.isIEMobile = agent.indexOf("msie 4")!=-1 || agent.indexOf("msie 5")!=-1;
 this.isOpera = typeof window.opera !== 'undefined';
@@ -178,8 +181,9 @@ function host(url) {
 }
 
 this.initAjaxComm = function(url, handler) {
-  var crossDomain = url.indexOf("://") != -1
-                    && host(url) != window.location.host;
+  var crossDomain = 
+    (url.indexOf("://") != -1 || url.indexOf("//") == 0) &&
+    host(url) != window.location.host;
 
   function createRequest(method, url) {
     var request = null;
@@ -470,6 +474,13 @@ this.unstub = function(from, to, methodDisplay) {
     to.style.width = from.style.width;
 };
 
+this.saveReparented = function(el) {
+  $(el).find('.wt-reparented').each(function() {
+      var domRoot = $('.Wt-domRoot').get(0);
+      domRoot.appendChild(this.parentNode.removeChild(this));
+    });
+};
+
 this.changeTag = function(e, type)
 {
   var n = document.createElement(type);
@@ -547,11 +558,16 @@ this.ajaxInternalPaths = function(basePath) {
 	el.innerHTML= '<a href="' + basePath +'">x</a>';
 	var absBase = el.firstChild.href;
 	internalPath = href.substr(absBase.length - 1);
-      } else
+      } else {
+	while (href.substr(0, 3) == "../")
+	  href = href.substr(3);
+	if (href.charAt(0) != '/')
+	  href = '/' + href;
 	internalPath = href.substr(basePath.length);
+      }
 
       if (internalPath.substr(0, 3) == "?_=")
-	  internalPath = internalPath.substr(3);
+	internalPath = internalPath.substr(3);
       this.setAttribute('href', href); // computes this.href
       this.setAttribute('href', this.href);
       this.onclick = function(event) {
@@ -562,8 +578,13 @@ this.ajaxInternalPaths = function(basePath) {
 };
 
 this.resolveRelativeAnchors = function() {
-  $('.Wt-rr').each(function() {
-      this.setAttribute('href', this.href);
+  if (window.$)
+    $('.Wt-rr').each(function() {
+      if (this.href)
+	this.setAttribute('href', this.href);
+      if (this.src)
+	this.setAttribute('src', this.src);
+
       $(this).removeClass("Wt-rr");
     });
 };
@@ -616,23 +637,6 @@ this.getElement = function(id) {
 
 this.$ = this.getElement;
 
-this.validate = function(edit) {
-  var v;
-  if (edit.options)
-    v = edit.options.item(edit.selectedIndex).text;
-  else
-    v = edit.value;
-
-  v = edit.wtValidate.validate(v);
-  if (v.valid) {
-    edit.removeAttribute('title');
-    $(edit).removeClass('Wt-invalid');
-  } else {
-    edit.setAttribute('title', v.message);
-    $(edit).addClass('Wt-invalid');
-  }
-};
-
 this.filter = function(edit, event, tokens) {
   var c = String.fromCharCode((typeof event.charCode !== 'undefined') ?
                               event.charCode : event.keyCode);
@@ -640,8 +644,9 @@ this.filter = function(edit, event, tokens) {
     WT.cancelEvent(event);
 };
 
-// Get coordinates of element relative to page origin.
-this.widgetPageCoordinates = function(obj) {
+// Get coordinates of element relative to an ancestor object (or page origin).
+// It computes the location of the left-top corner of the margin-box.
+this.widgetPageCoordinates = function(obj, reference) {
   var objX = 0, objY = 0, op;
 
   if (!obj.parentNode)
@@ -654,7 +659,7 @@ this.widgetPageCoordinates = function(obj) {
 
   var rtl = $(document.body).hasClass('Wt-rtl');
 
-  while (obj) {
+  while (obj && obj !== reference) {
     objX += obj.offsetLeft;
     objY += obj.offsetTop;
 
@@ -740,9 +745,27 @@ this.wheelDelta = function(e) {
 };
 
 this.scrollIntoView = function(id) {
-  var obj = document.getElementById(id);
-  if (obj && obj.scrollIntoView)
-    obj.scrollIntoView(true);
+  setTimeout(function() { 
+      var hashI = id.indexOf('#');
+      if (hashI != -1)
+	id = id.substr(hashI + 1);
+
+      var obj = document.getElementById(id);
+      if (obj) {
+	/* Locate a suitable ancestor to scroll */
+	var p;
+	for (p = obj.parentNode; p != document.body; p = p.parentNode) {
+	  if (p.scrollHeight > p.clientHeight &&
+	      WT.css(p, 'overflow-y') == 'auto') {
+	    var xy = WT.widgetPageCoordinates(obj, p);
+	    p.scrollTop += xy.y;
+	    return;
+	  }
+	}
+
+	obj.scrollIntoView(true);
+      }
+    }, 100);
 };
 
 this.getSelectionRange = function(elem) {
@@ -1092,8 +1115,8 @@ function initCapture() {
 
   captureInitialized = true;
 
-  if (document.body.addEventListener) {
-    var db = document.body;
+  var db = document.body;
+  if (db.addEventListener) {
     db.addEventListener('mousemove', mouseMove, true);
     db.addEventListener('mouseup', mouseUp, true);
 
@@ -1105,7 +1128,6 @@ function initCapture() {
 			      }, true);
     }
   } else {
-    var db = document.body;
     db.attachEvent('onmousemove', mouseMove);
     db.attachEvent('onmouseup', mouseUp);
   }
@@ -1247,8 +1269,8 @@ this.getCssRule = function(selector, deleteFlag) {
 	  }
 	} catch (err) {
 	  /*
-	     firefox security error 1000 when access a stylesheet.cssRules hosted
-	     from another domain
+	     firefox security error 1000 when access a stylesheet.cssRules
+	     hosted from another domain
 	   */
 	}
 
@@ -1303,46 +1325,53 @@ this.windowSize = function() {
  * bottom of (y) or top from (bottomy)
  */
 this.fitToWindow = function(e, x, y, rightx, bottomy) {
-  var ws = WT.windowSize();
+  var hsides = [ 'left', 'right' ],
+      vsides = [ 'top', 'bottom' ];
 
-  var wx = document.body.scrollLeft + document.documentElement.scrollLeft;
-  var wy = document.body.scrollTop + document.documentElement.scrollTop;
+  e.style[hsides[0]] = e.style[hsides[1]] = 'auto';
+  e.style[vsides[0]] = e.style[vsides[1]] = 'auto';
 
-  if (!e.offsetParent)
+  var elementWidth = WT.px(e, 'maxWidth') || e.offsetWidth,
+      elementHeight = WT.px(e, 'maxHeight') || e.offsetHeight,
+      hside, vside,
+      windowSize = WT.windowSize(),
+      windowX = document.body.scrollLeft + document.documentElement.scrollLeft,
+      windowY = document.body.scrollTop + document.documentElement.scrollTop;
+
+  var op = e.offsetParent;
+  if (!op)
     return;
 
-  var ow = WT.widgetPageCoordinates(e.offsetParent);
+  var offsetParent = WT.widgetPageCoordinates(op);
 
-  var hsides = [ 'left', 'right' ],
-      vsides = [ 'top', 'bottom' ],
-      ew = WT.px(e, 'maxWidth') || e.offsetWidth,
-      eh = WT.px(e, 'maxHeight') || e.offsetHeight,
-      hside, vside;
-
-  if (ew > ws.x) { // wider than window
-    x = wx;
+  if (elementWidth > windowSize.x) {
+    // wider than window
+    x = windowX;
     hside = 0;
-  } else if (x + ew > wx + ws.x) { // too far right, chose other side
-    rightx -= ow.x;
-    x = e.offsetParent.offsetWidth - (rightx + WT.px(e, 'marginRight'));
+  } else if (x + elementWidth > windowX + windowSize.x) {
+    // too far right, chose other side
+    rightx = rightx - offsetParent.x + op.scrollLeft;
+    x = op.clientWidth - (rightx + WT.px(e, 'marginRight'));
     hside = 1;
   } else {
-    x -= ow.x;
+    x = x - offsetParent.x + op.scrollLeft;
     x = x - WT.px(e, 'marginLeft');
     hside = 0;
   }
 
-  if (ew > ws.y) { // taller than window
-    y = wy;
+  if (elementHeight > windowSize.y) {
+    // taller than window
+    y = windowY;
     vside = 0;
-  } else if (y + eh > wy + ws.y) { // too far below, chose other side
-    if (bottomy > wy + ws.y)
-      bottomy = wy + ws.y;
-    bottomy -= ow.y;
-    y = e.offsetParent.offsetHeight - (bottomy + WT.px(e, 'marginBottom'));
+  } else if (y + elementHeight > windowY + windowSize.y) {
+    // too far below, chose other side
+    if (bottomy > windowY + windowSize.y)
+      bottomy = windowY + windowSize.y;
+    bottomy = bottomy - offsetParent.y + op.scrollTop;
+    y = op.clientHeight - (bottomy + WT.px(e, 'marginBottom'));
     vside = 1;
   } else {
-    y -= ow.y;
+    y = y - offsetParent.y + op.scrollTop;
     y = y - WT.px(e, 'marginTop');
     vside = 0;
   }
@@ -1355,46 +1384,41 @@ this.fitToWindow = function(e, x, y, rightx, bottomy) {
   */
 
   e.style[hsides[hside]] = x + 'px';
-  e.style[hsides[1 - hside]] = '';
   e.style[vsides[vside]] = y + 'px';
-  e.style[vsides[1 - vside]] = '';
 };
 
 this.positionXY = function(id, x, y) {
   var w = WT.getElement(id);
 
-  if (!WT.isHidden(w))
+  if (!WT.isHidden(w)) {
+    w.style.display = 'block';
     WT.fitToWindow(w, x, y, x, y);
+  }
 };
 
 this.Horizontal = 0x1;
 this.Vertical = 0x2;
 
-this.positionAtWidget = function(id, atId, orientation, parentInRoot,
-				 autoShow) {
+this.positionAtWidget = function(id, atId, orientation, delta) {
   var w = WT.getElement(id),
     atw = WT.getElement(atId);
 
+  if (!delta)
+    delta = 0;
+
   if (!atw || !w)
     return;
 
   var xy = WT.widgetPageCoordinates(atw),
     x, y, rightx, bottomy;
 
-  if (parentInRoot) {
-    w.parentNode.removeChild(w);
-    $('.Wt-domRoot').get(0).appendChild(w);
-  }
-
   w.style.position = 'absolute';
-  if (autoShow)
-    w.style.display = 'block';
 
-  if (orientation == WT.Horizontal) {
+  if (orientation === WT.Horizontal) {
     x = xy.x + atw.offsetWidth;
-    y = xy.y;
-    rightx = xy.x,
-    bottomy = xy.y + atw.offsetHeight;
+    y = xy.y + delta;
+    rightx = xy.x;
+    bottomy = xy.y + atw.offsetHeight - delta;
   } else {
     x = xy.x;
     y = xy.y + atw.offsetHeight;
@@ -1402,6 +1426,35 @@ this.positionAtWidget = function(id, atId, orientation, parentInRoot,
     bottomy = xy.y;
   }
 
+  /*
+   * Reparent the widget in a suitable parent:
+   *  an ancestor of w which isn't overflowing
+   */
+  if (!w.wtNoReparent) {
+    var p, pp = atw, domRoot = $('.Wt-domRoot').get(0);
+    w.parentNode.removeChild(w);
+  
+    for (p = pp.parentNode; p != domRoot; p = p.parentNode) {
+      if (p.wtResize) {
+	p = pp;
+	break;
+      }
+
+      if (p.scrollHeight > p.clientHeight || p.scrollWidth > p.clientWidth) {
+	break;
+      }
+
+      pp = p;
+    }
+
+    var posP = WT.css(p, 'position');
+    if (posP != 'absolute' && posP != 'relative')
+      p.style.position = 'relative';
+    
+    p.appendChild(w);
+    $(w).addClass('wt-reparented');
+  }
+
   WT.fitToWindow(w, x, y, rightx, bottomy);
 
   w.style.visibility='';
@@ -1409,12 +1462,50 @@ this.positionAtWidget = function(id, atId, orientation, parentInRoot,
 
 this.hasFocus = function(el) {
   try {
-    return el == document.activeElement;
+    return el === document.activeElement;
   } catch(e) {
     return false;
   }
 };
 
+this.progressed = function(domRoot) {
+  var doc = document, db = doc.body;
+  var form = this.getElement('Wt-form');
+
+  domRoot.style.display = form.style.display;
+  form.parentNode.replaceChild(domRoot, form);
+
+  if (db.removeEventListener)
+    db.removeEventListener('click', delayClick, true);
+  else
+    db.detachEvent('click', delayClick);
+
+  setTimeout(function() {
+      var i, il;
+      for (i = 0, il = delayedClicks.length; i < il; ++i) {
+	if (doc.createEvent) {
+	  var e = delayedClicks[i];
+	  var ec = doc.createEvent('MouseEvents');
+	  ec.initMouseEvent('click', e.bubbles, e.cancelable, window,
+			    e.detail, e.screenX, e.screenY,
+			    e.clientX, e.clientY, e.ctrlKey, e.altKey,
+			    e.shiftKey, e.metaKey, e.button, null);
+	  var el = WT.getElement(e.targetId);
+	  if (el)
+	    el.dispatchEvent(ec);
+	} else {
+	  var e = delayedClicks[i];
+	  var ec = doc.createEventObject();
+	  for (i in e)
+	    ec[i] = e[i];
+	  var el = WT.getElement(e.targetId);
+	  if (el)
+	    el.fireEvent('onclick', ec);
+	}
+      }
+    }, 0);
+};
+
 var html5History = !!(window.history && window.history.pushState);
 
 /*
@@ -1425,7 +1516,7 @@ function gentleURIEncode(s) {
   return s.replace(/%/g, '%25')
     .replace(/\+/g, '%2b')
     .replace(/ /g, '%20')
-    .replace(/#/g, '%23')
+    //.replace(/#/g, '%23')
     .replace(/&/g, '%26');
 }
 
@@ -1452,10 +1543,10 @@ if (html5History) {
       function onPopState(event) {
 	var newState = event.state;
 
-	if (newState === null)
+	if (newState == null)
 	  newState = stateMap[w.location.pathname + w.location.search];
 
-	if (newState === null) {
+	if (newState == null) {
 	  saveState(currentState);
 	  return;
 	}
@@ -1537,7 +1628,49 @@ _$_$endif_$_();
     }
   };
 })();
+} else if (WT.isIE8) {
+  this.history = (function()
+{
+  var currentState = null, cb = null, w = window;
 
+  return {
+    _initialize: function() { },
+
+    _initTimeout: function() { },
+
+    register: function (initialState, onStateChange) {
+      currentState = initialState;
+      cb = onStateChange;
+
+      function onHashChange() {
+	if (currentState != w.location.hash) {
+	  currentState = w.location.hash.substring(1);
+	  cb(currentState);
+	}
+      }
+
+      w.onhashchange = onHashChange;
+    },
+
+    initialize: function (stateField, histFrame, deployUrl) {
+    },
+
+    navigate: function (state, generateEvent) {
+      currentState = state;
+
+      w.location.hash = state;
+
+      WT.scrollIntoView(state);
+
+      if (generateEvent)
+	cb(state);
+    },
+
+    getCurrentState: function () {
+      return currentState;
+    }
+  };
+})();
 } else {
   this.history = (function()
 {
@@ -1549,7 +1682,6 @@ _$_$endif_$_();
    * http://developer.yahoo.net/yui/license.txt
    * version: 2.5.2
    */
-  var _UAwebkit = false;
   var _UAie = WT.isIElt9;
   var _UAopera = false;
   var _onLoadFn = null;
@@ -1568,9 +1700,6 @@ _$_$endif_$_();
   }
   function _storeStates() {
     _stateField.value = _initialState + "|" + _currentState;
-    if (_UAwebkit) {
-      _stateField.value += "|" + _fqstates.join(",");
-    }
   }
   function onStateChange() {
     var i, il;
@@ -1629,7 +1758,9 @@ _$_$endif_$_();
 	  } else {
 	    newHash = fqstate;
 	  }
-	  location.hash = newHash;
+	  if (location.hash != newHash && 
+	      location.hash.substring(1) != newHash)
+	    location.hash = newHash;
 	  hash = newHash;
 	  _storeStates();
 	} else if (newHash !== hash) {
@@ -1659,12 +1790,6 @@ _$_$endif_$_();
 	  counter = newCounter;
 	  _handleFQStateChange(hash);
 	  _storeStates();
-	} else if (newCounter !== counter && _UAwebkit) {
-	  hash = newHash;
-	  counter = newCounter;
-	  state = _fqstates[counter - 1];
-	  _handleFQStateChange(state);
-	  _storeStates();
 	}
       }, 50);
   }
@@ -1713,13 +1838,6 @@ _$_$endif_$_();
     if (vendor === "KDE") {
     } else if (typeof window.opera !== "undefined")
       _UAopera = true;
-    else if (!_UAie && vendor.indexOf("Apple Computer, Inc.") > -1)
-      _UAwebkit = true;
-
-    /*
-    if (_UAopera && typeof history.navigationMode !== "undefined")
-      history.navigationMode = "compatible";
-    */
 
     if (typeof stateField === "string")
       stateField = document.getElementById(stateField);
@@ -1751,11 +1869,8 @@ _$_$endif_$_();
     if (_UAie) {
       _updateIFrame(fqstate);
     } else {
-      if (fqstate.length > 0)
+      if (fqstate.length > 0) {
 	location.hash = fqstate;
-      if (_UAwebkit) {
-	_fqstates[history.length] = fqstate;
-	_storeStates();
       }
     }
     if (generateEvent)
@@ -1774,7 +1889,7 @@ _$_$endif_$_();
 
 })();
 
-if (window._$_APP_CLASS_$_) {
+if (window._$_APP_CLASS_$_ && window._$_APP_CLASS_$_._p_) {
   try {
     window._$_APP_CLASS_$_._p_.quit();
   } catch (e) {
@@ -1802,23 +1917,27 @@ var currentHash = null;
 function onHashChange() {
   var newLocation = _$_WT_CLASS_$_.history.getCurrentState();
 
-  if (newLocation.length > 0 && newLocation.substr(0, 1) != '/')
+  if (newLocation != null &&
+      newLocation.length > 0 &&
+      newLocation.substr(0, 1) != '/')
     return;
 
   if (currentHash == newLocation)
     return;
 
   currentHash = newLocation;
+
   setTimeout(function() { update(null, 'hash', null, true); }, 1);
 };
 
-function setHash(newLocation) {
-  if (currentHash == newLocation || !currentHash && newLocation == '/')
+function setHash(newLocation, generateEvent) {
+  if (currentHash == newLocation || (!currentHash && newLocation == '/'))
     return;
 
-  currentHash = newLocation;
+  if (!generateEvent)
+    currentHash = newLocation;
 
-  WT.history.navigate(newLocation, false);
+  WT.history.navigate(newLocation, generateEvent);
 };
 
 var dragState = {
@@ -1902,11 +2021,11 @@ function dragStart(obj, e) {
 };
 
 function dragDrag(e) {
-  if (dragState.object != null) {
+  if (dragState.object !== null) {
     var ds = dragState;
     var xy = WT.pageCoordinates(e);
 
-    if (ds.object.style["display"] != '' && ds.xy.x != xy.x && ds.xy.y != xy.y)
+    if (ds.object.style["display"] !== '' && ds.xy.x !== xy.x && ds.xy.y !== xy.y)
       ds.object.style["display"] = '';
 
     ds.object.style["left"] = (xy.x - ds.offsetX) + 'px';
@@ -2116,16 +2235,20 @@ function encodeEvent(event, i) {
     result += se + 'screenX=' + e.screenX + se + 'screenY=' + e.screenY;
 
   var widgetCoords = { x: 0, y: 0 };
+
   if (event.object && event.object.nodeType != 9) {
     widgetCoords = WT.widgetPageCoordinates(event.object);
     var objX = widgetCoords.x;
     var objY = widgetCoords.y;
 
-    result += se + 'scrollX=' + event.object.scrollLeft
-      + se + 'scrollY=' + event.object.scrollTop
-      + se + 'width=' + event.object.clientWidth
-      + se + 'height=' + event.object.clientHeight
-      + se + 'widgetX=' + (posX - objX) + se + 'widgetY=' + (posY - objY);
+    if (typeof event.object.scrollLeft != 'undefined') {
+      result += se + 'scrollX=' + event.object.scrollLeft
+	+ se + 'scrollY=' + event.object.scrollTop
+	+ se + 'width=' + event.object.clientWidth
+	+ se + 'height=' + event.object.clientHeight;
+    }
+
+    result += se + 'widgetX=' + (posX - objX) + se + 'widgetY=' + (posY - objY);
   }
 
   var button = WT.button(e);
@@ -2336,6 +2459,11 @@ function handleResponse(status, msg, timer) {
   if (quited)
     return;
 
+  if (waitingForJavaScript) {
+    setTimeout(function() { handleResponse(status, msg, timer); }, 50);
+    return;
+  }
+
   if (status == 0) {
     WT.resolveRelativeAnchors();
 _$_$if_CATCH_ERROR_$_();
@@ -2477,8 +2605,7 @@ _$_$if_WEB_SOCKETS_$_();
 	  } else {
 	    var query = sessionUrl.substr(sessionUrl.indexOf('?'));
 	    wsurl = "ws" + location.protocol.substr(4)
-	      + "//" + location.hostname + ":"
-	     + location.port + deployUrl + query;
+	      + "//" + location.host + deployUrl + query;
 	  }
 
 	  wsurl += "&request=ws";
@@ -2502,7 +2629,8 @@ _$_$if_WEB_SOCKETS_$_();
 	    /*
 	     * Sometimes, we can connect but cannot send data
 	     */
-	    if (reconnectTries == 3 && websocket.state == WebSocketsUnknown)
+	    if (websocket.reconnectTries == 3 &&
+		websocket.state == WebSocketsUnknown)
 	      websocket.state = WebSocketsUnavailable;
 	    reconnect();
 	  };
@@ -2511,8 +2639,8 @@ _$_$if_WEB_SOCKETS_$_();
 	    /*
 	     * Sometimes, we can connect but cannot send data
 	     */
-	    if (websocket.reconnectTries == 3
-		&& websocket.state == WebSocketsUnknown)
+	    if (websocket.reconnectTries == 3 &&
+		websocket.state == WebSocketsUnknown)
 	      websocket.state = WebSocketsUnavailable;
 	    reconnect();
 	  };
@@ -2528,15 +2656,20 @@ _$_$if_WEB_SOCKETS_$_();
 	     *
 	     * So, we ping pong ourselves. It costs virtually nothing.
 	     */
+	    ws.send('&signal=ping'); // to get our first onmessage
+
+	    if (websocket.keepAlive)
+	      clearInterval(websocket.keepAlive);
+
 	    websocket.keepAlive = setInterval
-	    (function() {
-	       if (ws.readyState == 1)
-		 ws.send('&signal=ping');
-	       else {
-		 clearInterval(websocket.keepAlive);
-		 websocket.keepAlive = null;
-	       }
-	     }, _$_SERVER_PUSH_TIMEOUT_$_);
+	      (function() {
+		if (ws.readyState == 1)
+		  ws.send('&signal=ping');
+		else {
+		  clearInterval(websocket.keepAlive);
+		  websocket.keepAlive = null;
+		}
+	      }, _$_SERVER_PUSH_TIMEOUT_$_);
 	  };
 	}
       }
@@ -2669,6 +2802,14 @@ function sendUpdate() {
 }
 
 function propagateSize(element, width, height) {
+  /*
+   * Propagate the size, even if it's the elements unconstrained size.
+   */
+  if (width == -1)
+    width = element.offsetWidth;
+  if (height == -1)
+    height = element.offsetHeight;
+
   if ((typeof element.wtWidth === 'undefined')
       || (element.wtWidth != width)
       || (typeof element.wtHeight === 'undefined')
@@ -2676,7 +2817,8 @@ function propagateSize(element, width, height) {
     element.wtWidth = width;
     element.wtHeight = height;
 
-    emit(element, 'resized', width, height);
+    if (width >= 0 && height >= 0)
+      emit(element, 'resized', width, height);
   }
 }
 
@@ -2734,22 +2876,28 @@ function addTimerEvent(timerid, msec, repeat) {
 	obj.onclick();
     }
   };
-
+  
   var obj = WT.getElement(timerid);
+  if (obj.timer)
+    clearTimeout(obj.timer);
   obj.timer = setTimeout(tm, msec);
   obj.tm = tm;
 }
 
 var jsLibsLoaded = {};
+var waitingForJavaScript = false;
 
 function onJsLoad(path, f) {
   // setTimeout needed for Opera
   setTimeout(function() {
     if (jsLibsLoaded[path] === true) {
+      waitingForJavaScript = false;
       f();
     } else
       jsLibsLoaded[path] = f;
     }, 20);
+
+  waitingForJavaScript = true;
 };
 
 function jsLoaded(path)
@@ -2757,8 +2905,10 @@ function jsLoaded(path)
   if (jsLibsLoaded[path] === true)
     return;
   else {
-    if (typeof jsLibsLoaded[path] !== 'undefined')
+    if (typeof jsLibsLoaded[path] !== 'undefined') {
+      waitingForJavaScript = false;
       jsLibsLoaded[path]();
+    }
     jsLibsLoaded[path] = true;
   }
 };
@@ -2836,6 +2986,68 @@ ImagePreloader.prototype.onload = function() {
     preloader.callback(preloader.images);
 };
 
+/////////////////////////////////////////////////////////////////////
+// TG: A binary Buffer preloader
+
+// Constructor, preloads the given uris and stores them in arrayBuffers[]
+function ArrayBufferPreloader(uris, callback) {
+  // init members
+  // callback, when everything is loaded
+  this.callback = callback;
+  // number of open requests
+  this.work = uris.length;
+  // resulting buffers
+  this.arrayBuffers = [];
+  
+  // if urls are missing, call callback without buffers
+  if (uris.length == 0)
+    callback(this.arrayBuffers);
+  else 
+  {
+    // if uris are given, load them asynchronously
+    for (var i = 0; i < uris.length; i++)
+      this.preload(uris[i], i);
+  }
+};
+
+// preload function: downloads buffer at the given URI
+ArrayBufferPreloader.prototype.preload = function(uri, index) {
+  var xhr = new XMLHttpRequest();
+  // open the resource, send asynchronously (without waiting for answer)
+  xhr.open("GET", uri, true);
+  xhr.responseType = "arraybuffer"; 
+
+  // give xhr write access to array
+  xhr.arrayBuffers = this.arrayBuffers;
+  xhr.preloader = this;
+  xhr.index = index; // needed to maintain the mapping
+  xhr.uri = uri;
+
+  // behaviour when it was loaded-> redirect to ArrayBufferPreloader
+  xhr.onload = function(e) {
+
+    console.log("XHR load buffer " + this.index + " from uri " + this.uri);
+
+    //this.arrayBuffers[this.index] = new Uint8Array(this.response);
+    this.arrayBuffers[this.index] = this.response;
+    this.preloader.afterLoad();
+  };
+
+  xhr.onerror = ArrayBufferPreloader.prototype.afterload;
+  xhr.onabort = ArrayBufferPreloader.prototype.afterload;
+
+  // actually start the query
+  xhr.send();
+};
+
+ArrayBufferPreloader.prototype.afterLoad = function() {
+  if (--this.work == 0)
+    // last request finished -> call callback
+    this.callback(this.arrayBuffers);
+};
+/////////////////////////////////////////////////////////////////////
+
+
 function enableInternalPaths(initialHash) {
   currentHash = initialHash;
   WT.history.register(initialHash, onHashChange);
@@ -2900,15 +3112,16 @@ this._p_ = {
   enableInternalPaths : enableInternalPaths,
   onHashChange : onHashChange,
   setHash : setHash,
-  ImagePreloader : ImagePreloader,
-
+  ImagePreloader : ImagePreloader,  
+  ArrayBufferPreloader : ArrayBufferPreloader,
+  
   doAutoJavaScript : doAutoJavaScript,
   autoJavaScript : function() { },
 
   response : responseReceived,
   setPage : setPage,
   setCloseMessage : setCloseMessage,
-
+  
   propagateSize : propagateSize
 };
 
